from PIL import Image


def topng(imgPath: str):
    """
    将图片路径所指的图片转为jpg格式

    transform image to jpg

    :param imgPath: 图片路径 | image path
    :return:
    """
    Image.open(imgPath).save(imgPath + ".png", quality=100)


def tojpg(imgPath: str):
    """
    将图片路径所指的图片转为jpg格式

    transform image to jpg

    :param imgPath: 图片路径 | image path
    :return:
    """
    Image.open(imgPath).convert("RGB").save(imgPath + ".jpg", quality=100)



# coding=utf-8
"""
七牛云对象存储API

Qiniu cloud OSS API
"""
from . import pre_check, dir_char, user_lang
from .. import requirePackage


class QiniuOSS:
    qiniu = requirePackage("qiniu")

    def __init__(
        self,
        ac_key: str = pre_check("qiniu_ac_key"),
        sc_key: str = pre_check("qiniu_sc_key"),
        df_bucket: str = pre_check("qiniu_bk_name"),
    ):
        """
        初始化并登陆七牛云对象存储

        Initializes and logs in qiniu cloud object storage

        :param ac_key: Access Key
        :param sc_key: Secret Key
        :param df_bucket: 默认桶名称
        """
        self.ac_key = ac_key
        self.sc_key = sc_key
        self.auth = QiniuOSS.qiniu.Auth(self.ac_key, self.sc_key)
        self.df_bucket = df_bucket
        self.bar, self.tid = None, None

    def get_func_table(self):
        """
        获取对象支持的操作

        Gets the operations supported by the object

        :return: 函数表
        """
        return {
            "-up": self.upload,
            "-rm": self.remove,
            "-cp": self.copy_url,
            "-dl": self.download,
            "-ls": self.list_bucket,
        }

    def upload(self, filePath: str, bucket: str = None):
        """
        上传文件

        Upload file

        :param filePath: 文件路径
        :param bucket: 桶名称，缺省使用self.df_bucket
        :return: None
        """
        import os

        filename = os.path.basename(filePath)

        def progress(cur, total):
            if not self.bar:
                from ..TuiTools.Bar import DataTransformBar

                self.bar = DataTransformBar()
                self.tid = self.bar.add_task(
                    "Transform" if user_lang != "zh" else "传输",
                    filename=filename,
                    total=total,
                )
                self.bar.start()
                self.bar.start_task(self.tid)
            self.bar.update(self.tid, completed=cur)

        filePath = filePath.strip()
        tk = self.auth.upload_token(bucket if bucket else self.df_bucket, filePath)
        QiniuOSS.qiniu.put_file(
            tk, filePath.replace(dir_char, "/"), filePath, progress_handler=progress
        )
        self.bar.stop()

    def remove(self, filePath: str, bucket: str = None):
        """
        删除文件

        Delete file

        :param filePath: 文件路径（对象存储中）
        :param bucket: 桶名称，缺省使用self.df_bucket
        :return: None
        """
        bk = QiniuOSS.qiniu.BucketManager(self.auth)
        bk.delete(bucket if bucket else self.df_bucket, filePath)

    def copy_url(self, filePath: str, bucket: str = None):
        """
        通过url拷贝文件（这个接口貌似没有卵用，七牛云那边并不会生效）

        Copy files through URL (this interface seems to have no egg use, qiniu cloud will not work)

        :param filePath: 文件链接
        :param bucket: 桶名称，缺省使用self.df_bucket
        :return: None
        """
        bk = QiniuOSS.qiniu.BucketManager(self.auth)
        bk.fetch(
            filePath, bucket if bucket else self.df_bucket, filePath.split("/")[-1]
        )

    def get_bucket_url(self, bucket: str = None):
        """
        获取当前桶的访问链接

        Gets the access link for the current bucket

        :param bucket: 桶名称，缺省使用self.df_bucket
        :return: 成功返回json，否则False
        """
        import requests

        bucket = bucket if bucket else self.df_bucket
        url = "http://api.qiniu.com/v6/domain/list?tbl=%s" % bucket
        res = requests.get(
            url,
            headers={
                "Content-Type": "application/x-www-form-urlencoded",
                "Authorization": "QBox %s" % self.auth.token_of_request(url),
            },
        )
        if res.status_code == requests.codes.ok:
            return res.json()
        else:
            return False

    def list_bucket(self, bucket: str = None):
        """
        展示bucket中所有的文件

        Displays all the files in the bucket

        :param bucket: 桶名称，缺省使用self.df_bucket
        :return: None
        """
        from .. import qs_default_console, qs_info_string
        from ..TuiTools.Table import qs_default_table
        from rich.text import Text

        bk = QiniuOSS.qiniu.BucketManager(self.auth)
        ret = bk.list(bucket if bucket else self.df_bucket)
        if not ret[1]:
            print("ERROR!" if user_lang != "zh" else "错误!")
            exit(0)
        root_url = "http://" + self.get_bucket_url(bucket)[0] + "/"
        ret = ret[0]["items"]
        from ..NetTools.NormalDL import size_format

        tb = qs_default_table(
            ["File", "Size"] if user_lang != "zh" else ["文件", "体积"], title="七牛 OSS"
        )
        for i in ret:
            tb.add_row(
                Text(i["key"], justify="left"),
                Text(size_format(i["fsize"], True), justify="right"),
            )
        qs_default_console.print(
            qs_info_string, "Bucket url:" if user_lang != "zh" else "桶链接:", root_url
        )
        qs_default_console.print(tb, justify="center")

    def download(self, filePath: str, bucket: str = None):
        """
        下载文件

        Download file

        :param filePath: 文件在桶中的路径
        :param bucket: 桶名称，缺省使用self.df_bucket
        :return: None
        """
        from ..NetTools.NormalDL import normal_dl

        bucket = bucket if bucket else self.df_bucket
        root_url = self.get_bucket_url(bucket)[0]
        if root_url:
            root_url = "http://" + root_url + "/"
        else:
            exit("Get Bucket Url Failed!" if user_lang != "zh" else "获取桶链接失败!")
        dl_url = root_url + filePath
        if bucket.startswith("admin"):
            dl_url = self.auth.private_download_url(dl_url)
        normal_dl(dl_url)



# coding=utf-8
"""
柱状图

histogram & Bar
"""
from colorama import Back, Style
import math


class RollBar:
    def __init__(self, values: list, max_: int = 100, height: int = 10):
        """
        滚动柱状图初始化

        Rolling histogram initialization

        :param values: 值列表
        :param max_: 最大值
        :param height: 高度
        """
        self._title = ""
        self.max_ = max_
        self.width = len(values)
        self.height = height
        self.canvas = [[] for ignore in range(height)]
        for val in values:
            cur = math.ceil(val / max_ * height)
            for _i in range(cur):
                self.canvas[_i].append(Back.GREEN + " " + Style.RESET_ALL)
            for _i in range(cur, height):
                self.canvas[_i].append(Style.RESET_ALL + " ")

    def __str__(self):
        """
        返回字符串形式的图

        Returns a graph as a string

        :return: 字符串形式的图
        """
        if self.title:
            title_idx = (self.width - len(self._title)) // 2 + 1
            title_idx = max(title_idx, 0)
            res = " " * title_idx + self._title + "\n"
        else:
            res = ""
        return (
            res
            + "100%%%s" % ("_" * (self.width + 1))
            + "\n    │"
            + "\n    │".join(["".join(i) for i in self.canvas[::-1]])
            + "\n  0%╂"
            + "────┴" * math.ceil(self.width // 5)
        )

    def show(self):
        """
        打印柱状图

        Print the bar chart

        :return: None
        """
        print(self)

    def _fresh(self, val):
        cur = math.ceil(val / self.max_ * self.height)
        for _i in range(cur):
            self.canvas[_i].append(Back.GREEN + " " + Style.RESET_ALL)
        for _i in range(cur, self.height):
            self.canvas[_i].append(Style.RESET_ALL + " ")

    def add(self, val: float):
        """
        扔掉第一个值，并在尾部添加val

        Drop the first value and add Val at the end

        :param val: 值
        :return: None
        """
        for _i in self.canvas:
            _i.pop(0)
        self._fresh(val)

    def title(self, _title: str):
        """
        设置柱状图的标题

        Sets the title of the bar chart

        :param _title: 标题
        :return: None
        """
        self._title = _title

    def set(self, vals: list):
        """
        设置柱状图的全部值

        Sets all values of the histogram

        :param vals: 值列表
        :return: None
        """
        for i in self.canvas:
            i.clear()
        for val in vals:
            self._fresh(val)


def DataTransformBar(has_size_info: bool = True):
    """
    创建一个数据传输进度条

    Create a data transfer progress bar

    :param has_size_info: 是否有总任务量
    :return: rich.progress.Progress
    """
    from rich.progress import (
        Progress,
        TextColumn,
        BarColumn,
        DownloadColumn,
        TransferSpeedColumn,
        TimeRemainingColumn,
    )
    from .. import qs_default_console

    if has_size_info:
        return Progress(
            TextColumn("[bold blue]{task.fields[filename]}", justify="right"),
            BarColumn(bar_width=None),
            "[progress.percentage]{task.percentage:>3.1f}%",
            "•",
            DownloadColumn(),
            "•",
            TransferSpeedColumn(),
            "•",
            TimeRemainingColumn(),
            console=qs_default_console,
        )
    else:
        from .. import user_lang

        return Progress(
            TextColumn(
                "[bold blue]{task.fields[filename]} [red]"
                + ("Unknow size" if user_lang != "zh" else "未知大小"),
                justify="right",
            ),
            BarColumn(bar_width=None),
            DownloadColumn(),
            console=qs_default_console,
        )


def NormalProgressBar(task_name, total):
    from .. import qs_default_console
    from rich.progress import Progress

    progress = Progress()
    task_id = progress.add_task(task_name, total=total, console=qs_default_console)
    return progress, task_id



from prompt_toolkit.validation import Validator, ValidationError
from inquirer_rhy.prompt import prompt
import sys
from rich.console import Console

qs_default_console = Console()
system = sys.platform
if system.startswith("win"):
    dir_char = "\\"
else:
    dir_char = "/"


default_language = {
    "type": "input",
    "name": "default_language",
    "message": """Select your language, the flowing content is available choice
  选择你的语言，下述内容为合法选项 (此处顺序无任何意义)
    
    zh  (Chinese) en  (English)  jp  (Japanese) kor (Korean)   fra (French), 
    spa (Spanish) th  (Thailand) ara (al-ummah) ru  (Russian)  pt  (Portuguese), 
    de  (Germany) it  (Italy)    el  (Greece)   nl  (Poland)   bul (Bulgaria),
    est (Estonia) dan (Denmark)  fin (Finland)  cs  (Czech)    rom (Romania),
    slo (Iceland) swe (Sweden)   hu  (Hungary)  vie (Vietnam)
    
  Input the default language | 输入默认语言:""",
    "validate": lambda val: val
    in [
        "zh",
        "en",
        "jp",
        "kor",
        "fra",
        "spa",
        "th",
        "ara",
        "ru",
        "pt",
        "de",
        "it",
        "el",
        "nl",
        "bul",
        "est",
        "dan",
        "fin",
        "cs",
        "rom",
        "slo",
        "swe",
        "hu",
        "vie",
        "Not Set | 暂不配置",
    ],
    "default": "Not Set | 暂不配置",
}

default_currency = {
    "type": "input",
    "name": "default_currency",
    "message": """Choose your common currency, the flowing content is available choice
  选择你常用的币种，下述内容为合法选项 (此处顺序无任何意义)
    
    CNY (Chinese)  USD (American) JPY (Japanese)  KRW (Korean)    DKK (Denmark)  EUR (Europe) 
    THB (Thailand) SAR (al-ummah) RUB (Russian)   BYR (Belarus)   RON (Romania)  PLN (Poland)
    BGN (Bulgaria) CZK (Czech)    ISK (Iceland)   VND (Vietnam)   DZD (Algeria)  ARS (Argentina)
    SEK (Sweden)   HUF (Hungary)  OMR (Amani)     AUD (Australia) MOP (Macao)    AED (United Arab Emirates)
    EGP (Egypt)    BHD (Bahrain)  BRL (Brazil)    HKD (HongKong)  COP (Colombia) PHP (Philippines)
    CAD (Canada)   KHR (Cambodia) QAR (Qatar)     HRK (Croatia)   KES (Kenya)    CRC (CostaRica)
    MXN (Mexico)   MAD (Morocco)  KWD (Kuwait)    LAK (Laos)      LBP (Lebanon)  MYR (Malaysia)
    BUK (Myanmar)  NOK (Norway)   SEK (Sweden)    RSD (Serbia)    TZS (Tanzania) ZAR (South Africa)
    BND (Brunei)   UGX (Uganda)   ZMK (Zambian)   SGD (Singapore) TWD (TaiWan)   LKR (Sri Lanka)  
    TRY (Turkey)   HUF (Hungary)  SYP (Syria)     IQD (Iraq)      INR (India)    NZD (New Zealand)
    GBP (England)  ILS (Israel)   JOD (Jordan)    CLP (Chile)     IDR (Indonesia)
    
  Input the default currency | 输入默认币种:""",
    "validate": lambda val: val
    in {
        "CLP",
        "AED",
        "CZK",
        "THB",
        "MYR",
        "NZD",
        "LBP",
        "LAK",
        "HUF",
        "VND",
        "ZMK",
        "RSD",
        "CNH",
        "BYR",
        "HRK",
        "CHF",
        "CNY",
        "TWD",
        "CAD",
        "RON",
        "MOP",
        "CRC",
        "COP",
        "LKR",
        "IDR",
        "AUD",
        "ARS",
        "BGN",
        "KRW",
        "TZS",
        "JOD",
        "HKD",
        "EGP",
        "KHR",
        "ZAR",
        "BRL",
        "OMR",
        "BHD",
        "NOK",
        "PLN",
        "QAR",
        "RUB",
        "MAD",
        "EUR",
        "GBP",
        "BND",
        "SAR",
        "USD",
        "KWD",
        "SYP",
        "DKK",
        "ILS",
        "ISK",
        "DZD",
        "JPY",
        "SEK",
        "TRY",
        "INR",
        "KES",
        "SGD",
        "UGX",
        "PHP",
        "IQD",
        "BUK",
        "MXN",
        "Not set | 暂不设置",
    },
    "default": "Not set | 暂不设置",
}

default_translate_engine = {
    "type": "list",
    "name": "default_translate_engine",
    "message": "Select translate engine    | 选择翻译引擎:",
    "choices": ["default", "TencentCloud"],
}

default_pip = {
    "type": "input",
    "name": "default_pip",
    "message": "Input the default pip | 输入默认pip:",
    "default": "pip3",
}


force_show_img = {
    "type": "confirm",
    "name": "force_show_img",
    "message": "Force show image in terminal | 强制在终端显示图片?",
    "default": False,
}


class proxyValidator(Validator):
    from .NetTools import is_ip

    def validate(self, document):
        document = document.text
        if document == "Not set | 暂不设置":
            return True
        ip_flag = proxyValidator.is_ip(
            ":".join(document.split("@")[-1].split(":")[:-1])
        )
        if not ip_flag:
            raise ValidationError(
                message="Not a valid IP address | 非合法ip地址",
                cursor_position=len(document),
            )
        return False


default_proxy = {
    "type": "input",
    "name": "default_proxy",
    "message": "Input download proxy       | 输入下载代理:",
    "default": "Not set | 暂不设置",
    "validate": proxyValidator,
}


terminal_font_size = {
    "type": "input",
    "name": "terminal_font_size",
    "message": "Input terminal font size   | 输入终端字体大小:",
    "default": "16",
}


class QsConfig:
    import json

    def __init__(self, configPath, isExists):
        self.path = configPath
        if isExists:
            try:
                with open(configPath, "r") as f:
                    self.config = QsConfig.json.loads(f.read())
            except:
                with open(configPath, "r", encoding="utf8") as f:
                    self.config = QsConfig.json.loads(f.read(), encoding="utf8")
        else:
            self.config = QsConfig.json.loads(
                """{
  "basic_settings": {
    "default_language": "zh",
    "default_currency": "CNY",
    "default_translate_engine": {
      "index": 0,
      "support": ["default", "TencentCloud"]
    },
    "default_proxy": "user:password@ip:port or ip:port",
    "default_pip": "pip3",
    "force_show_img": false,
    "terminal_font_size": 16
  },
  "API_settings": {
    "rmbg": "GET: https://www.remove.bg",
    "smms": "GET: https://sm.ms",
    "darksky": "GET: https://darksky.net/",
    "aliyun_oss_acid": "GET: https://www.aliyun.com/product/oss",
    "aliyun_oss_ackey": "GET: https://www.aliyun.com/product/oss",
    "aliyun_oss_bucket_url": "GET: https://www.aliyun.com/product/oss",
    "aliyun_oss_df_bucket": "GET: https://www.aliyun.com/product/oss",
    "txyun_scid": "GET: https://console.cloud.tencent.com/",
    "txyun_sckey": "GET: https://console.cloud.tencent.com/",
    "txyun_cos_df_bucket": "GET: https://console.cloud.tencent.com/",
    "txyun_df_region": "GET: ap-[location]",
    "qiniu_ac_key": "GET: http://qiniu.com/",
    "qiniu_sc_key": "GET: http://qiniu.com/",
    "qiniu_bk_name": "GET: [Qiniu Bucket Name]",
    "gitee": "GET: http://gitee.com/",
    "AipImageAPP_ID": "GET: https://cloud.baidu.com/product/imageprocess",
    "AipImageAPP_KEY": "GET: https://cloud.baidu.com/product/imageprocess",
    "AipImageSECRET_KEY": "GET: https://cloud.baidu.com/product/imageprocess",
    "AipNlpAPP_ID" : "GET: https://cloud.baidu.com/product/nlp_apply",
    "AipNlpAPP_KEY": "GET: https://cloud.baidu.com/product/nlp_apply",
    "AipNlpSECRET_KEY": "GET: https://cloud.baidu.com/product/nlp_apply",
    "commonClipboardFilePath": "GET: /Path/to/file",
    "alapi_token": "GET: https://user.alapi.cn/",
    "lolicon_token": "GET: https://api.lolicon.app/#/setu?id=apikey",
    "openai": "GET: https://openai.com/api/"
  }
}"""
            )
            res = prompt(
                [
                    default_language,
                    default_currency,
                    default_translate_engine,
                    default_proxy,
                    default_pip,
                    terminal_font_size,
                ]
            )
            self.config["basic_settings"]["default_language"] = (
                res["default_language"]
                if res["default_language"] != "Not Set | 暂不配置"
                else "en"
            )
            self.config["basic_settings"]["default_currency"] = (
                res["default_currency"]
                if res["default_currency"] != "Not Set | 暂不配置"
                else "USD"
            )
            self.config["basic_settings"]["default_translate_engine"]["index"] = [
                "default",
                "TencentCloud",
            ].index(res["default_translate_engine"])
            self.config["basic_settings"]["default_proxy"] = (
                "" if res["default_proxy"] == "Not set | 暂不设置" else res["default_proxy"]
            )
            self.config["basic_settings"]["default_pip"] = res["default_pip"]
            self.update()
            qs_default_console.print(
                "\nYour configuration table has been stored\n你的配置表被存储在: [bold green]"
                + "%s" % configPath
                + "[/bold green]"
            )
            qs_default_console.print(
                "[bold red]\nqs will not use your configuration do anything!\nQpro不会用您的配置表做任何事情![/bold red]"
            )
            prompt(
                {
                    "type": "confirm",
                    "message": "Confirm | 确认",
                    "name": "done",
                    "default": True,
                }
            )
            if (
                system.startswith("darwin")
                and prompt(
                    {
                        "type": "confirm",
                        "name": "use_iTerm",
                        "message": """Qs recommends that you use iTerm as the terminal program in
  the Mac system, whether to open the iTerm2 official website?
  qs推荐您在Mac系统中使用iTerm2作为终端程序, 是否打开iTerm2官网?""",
                        "default": True,
                    }
                )["use_iTerm"]
            ):
                from .NetTools import open_url

                open_url("https://www.iterm2.com/")

    def update(self):
        with open(self.path, "w") as f:
            QsConfig.json.dump(self.config, f, indent=4, separators=(",", ": "))

    def basicSelect(self, key: str):
        if key not in self.config["basic_settings"]:
            exec(f"res = prompt([{key}])")
            exec(f"self.config['basic_settings']['{key}'] = res['{key}']")
            self.update()
        return self.config["basic_settings"][key]

    def apiSelect(self, key):
        if key not in self.config["API_settings"]:
            raise KeyError
        return self.config["API_settings"][key]

    def basicUpdate(self, key: str, value: str):
        self.config["basic_settings"][key] = value
        return self.update()

    def apiUpdate(self, key: str, value: str):
        self.config["API_settings"][key] = value
        return self.update()



from .__config__ import dir_char


class QsCache:
    """
    qs存储在~/.qs_cache的缓存
    """

    import os
    import pickle

    def __init__(self, cachePath: str):
        self.path = cachePath + dir_char
        if not QsCache.os.path.exists(cachePath):
            QsCache.os.mkdir(cachePath)

    def get(self, key: str):
        if not QsCache.os.path.exists(self.path + key):
            return None
        with open(self.path + key, "rb") as f:
            return QsCache.pickle.loads(f.read())

    def set(self, key: str, value):
        with open(self.path + key, "wb") as f:
            QsCache.pickle.dump(value, f)

    def delete(self, key: str):
        if QsCache.os.path.exists(self.path + key):
            QsCache.os.remove(self.path + key)



# coding=utf-8
"""
命令入口

Command entry
"""
import sys
from .funcList import *
from . import requirePackage


def qs_help(rep=""):
    """输出菜单 | Output menu"""
    if not rep or rep not in menu_table:
        if user_lang != "zh":
            color_rep(
                """
    qs basic   :-> basic   tools help | u, a, f, cal, time, pcat..
    qs system  :-> system  tools help | top, \[mk/un]\[zip/tar/7z]..
    qs net     :-> network tools help | http, dl, netinfo, wifi...
    qs api     :-> api     tools help | trans, smms, rmbg, loli...
    qs image   :-> image   tools help | stbg, v2gif, v2mp3, v2mp4.""",
                True,
            )
            qs_default_console.print(
                "\n[underline]Tutorial[/underline]\n", justify="center"
            )
            qs_default_console.print(
                "[bold magenta]qs lesson[/bold magenta]", justify="center"
            )
            qs_default_console.print("\n[underline]Docs\n", justify="center")
            qs_default_console.print(
                "C D N : https://rhythmlian.cn/2020/02/14/QuickStart-Rhy/ _ _ _",
                justify="center",
            )
            qs_default_console.print(
                "Github: https://rhythmicc.github.io/2020/02/14/QuickStart-Rhy/",
                justify="center",
            )
            qs_default_console.print("\n[underline]TG Group:\n", justify="center")
            qs_default_console.print(
                "https://t.me/joinchat/G2mpk7-S85eM7sb7", justify="center"
            )
        else:
            color_rep(
                """
    qs basic   :-> 基础工具帮助 | u, a, f, cal, time, pcat..
    qs system  :-> 系统工具帮助 | top, \[mk/un]\[zip/tar/7z]..
    qs net     :-> 网络工具帮助 | http, dl, netinfo, wifi...
    qs api     :-> 扩展工具帮助 | trans, smms, rmbg, loli...
    qs image   :-> 图像工具帮助 | stbg, v2gif, v2mp3, v2mp4.""",
                True,
            )
            qs_default_console.print("\n[underline]引导[/underline]\n", justify="center")
            qs_default_console.print(
                "[bold magenta]qs lesson[/bold magenta]", justify="center"
            )
            qs_default_console.print("\n[underline]文档[/underline]\n", justify="center")
            qs_default_console.print(
                "C D N : https://rhythmlian.cn/2020/08/09/QuickStart-Rhy-zh/ _ _ _",
                justify="center",
            )
            qs_default_console.print(
                "Github: https://rhythmicc.github.io/2020/08/09/QuickStart-Rhy-zh/",
                justify="center",
            )
            qs_default_console.print("\n[underline]TG群[/underline]\n", justify="center")
            qs_default_console.print(
                "https://t.me/joinchat/G2mpk7-S85eM7sb7", justify="center"
            )
    else:
        menu_table[rep]()


def main():
    """执行命令 | Execute"""
    debug_flag = "--qs-debug" in sys.argv
    if debug_flag:
        sys.argv.remove("--qs-debug")
    if len(sys.argv) >= 2:
        try:
            func_name = sys.argv[1]
            cmd_config = {
                j: i
                for i in [basic_funcs, api_funcs, net_funcs, image_funcs, system_funcs]
                for j in i
            }
            cmd_config.pop("self")
            if func_name not in cmd_config:
                qs_help(func_name)
            else:
                func_table = cmd_config[func_name]
                file_name = func_table["self"]
                func_name = func_table[func_name]
                (
                    requirePackage("QuickStart_Rhy", func_name)
                    if file_name == "basic"
                    else requirePackage(f"QuickStart_Rhy.{file_name}", func_name)
                )()
        except Exception as e:
            from . import qs_default_console

            if debug_flag:
                qs_default_console.print_exception()
            else:
                from . import qs_error_string

                qs_default_console.log(qs_error_string, repr(e))
    else:
        qs_help()


if __name__ == "__main__":
    main()



# coding=utf-8
"""
压缩与解压缩各种文件

Compress and decompress all kinds of files
"""
import io
import os
import sys
import tarfile
import zipfile
import rarfile
import py7zr
from .. import dir_char, user_lang, qs_default_console, qs_warning_string


def get_compress_package_name():
    """
    从命令行参数中，解析并返回压缩包名称

    From the command line arguments, the package name is parsed and returned.

    :return: 压缩包名称，合法的文件列表 | Compressed package name, a list of legitimate files.
    """
    file_names = sys.argv[2:]
    if not file_names:
        exit("No enough parameters")
    if len(file_names) > 1:
        tar_name = "pigeonhole"
    else:
        ls = file_names[0].split(dir_char)
        while not ls[-1]:
            ls.pop()
        tar_name = ls[-1].split(".")[0]
    ls = []
    for file_name in file_names:
        if os.path.exists(file_name):
            ls.append(file_name)
        else:
            print("No such file or dictionary:%s" % file_name)
    return tar_name, ls


def checkIsProtocolFile(protocol, path):
    """
    验证压缩包文件

    Verify compressed package file

    :param protocol: 压缩协议 | compress protocol
    :param path: 文件路径 | File Path
    :return:
    """
    if os.path.exists(path):
        if protocol == tarfile and not tarfile.is_tarfile(path):
            raise TypeError(
                f"{path} "
                + (
                    "Not recognized by tar protocol"
                    if user_lang != "zh"
                    else "无法被tar协议识别"
                )
            )
        elif protocol == zipfile and not zipfile.is_zipfile(path):
            raise TypeError(
                f"{path} "
                + (
                    "Not recognized by zip protocol"
                    if user_lang != "zh"
                    else "无法被zip协议识别"
                )
            )
        elif protocol == rarfile and not rarfile.is_rarfile(path):
            raise TypeError(
                f"{path} "
                + (
                    "Not recognized by rar protocol"
                    if user_lang != "zh"
                    else "无法被rar协议识别"
                )
            )
        elif protocol == py7zr and not py7zr.is_7zfile(path):
            raise TypeError(
                f"{path} "
                + (
                    "Not recognized by 7z protocol"
                    if user_lang != "zh"
                    else "无法被7z协议识别"
                )
            )
    else:
        raise FileNotFoundError


class _NormalCompressedPackage:
    """
    通用压缩协议类，如果你不懂它是做什么的，请不要调用它

    General compression protocol class, if you do not understand what it does, please do not call it
    """

    def __init__(self, _protocol, path: str, mode="r"):
        """
        通用压缩协议类初始化

        General compression protocol class initialization

        :param _protocol: 压缩协议包 | General compression protocol packages
        :param path: 压缩包路径 | compression package path
        :param mode: 读写模式 | 'r' or 'w', which 'r' means read and 'w' means write
        """
        self._protocol = _protocol
        self.path = path
        if mode not in ("r", "w"):
            raise ValueError("Requires mode 'r', 'w'")
        if mode == "r":
            checkIsProtocolFile(_protocol, path)
            if _protocol == zipfile:
                self.src = zipfile.ZipFile(path, "r")
            elif _protocol == rarfile:
                self.src = rarfile.RarFile(path)
            elif _protocol == py7zr:
                self.src = py7zr.SevenZipFile(path, "r")
            else:
                self.src = _protocol.open(path, "r")
            self.mode = True
        elif mode == "w":
            if _protocol == tarfile:
                self.src = _protocol.open(path, "x:gz")
            elif _protocol == zipfile:
                self.src = _protocol.ZipFile(path, "w")
            elif _protocol == rarfile:
                raise NotImplementedError(
                    "qs not support to create rar file because `RarFile`"
                )
            elif _protocol == py7zr:
                self.src = _protocol.SevenZipFile(path, "w")
            self.mode = False

    def add_file(self, path):
        """
        向压缩包添加文件（工作在'w'模式下）

        Add file to the compressed package (works in 'w' mode)

        :param path: 文件路径 | File path
        :return: None
        """
        if self.mode:
            raise io.UnsupportedOperation
        if not os.path.exists(path):
            raise FileNotFoundError
        if os.path.isdir(path):
            for root, dirs, files in os.walk(path):
                for file in files:
                    if self._protocol == tarfile:
                        self.src.add(os.path.join(root, file))
                    elif self._protocol in [zipfile, py7zr]:
                        self.src.write(os.path.join(root, file))
        else:
            if self._protocol == tarfile:
                self.src.add(path)
            elif self._protocol in [zipfile, py7zr]:
                self.src.write(path)

    def extract(self):
        """
        解压缩 | extract

        :return: None
        """
        if self.mode:
            if self._protocol in [tarfile, rarfile, py7zr]:
                self.src.extractall()
            elif self._protocol in [zipfile]:
                from pathlib import Path

                for fn in self.src.namelist():
                    path = Path(self.src.extract(fn))
                    try:
                        path.rename(fn.encode("cp437").decode("utf-8"))
                    except:
                        try:
                            path.rename(fn.encode("cp437").decode("gbk"))
                        except Exception as e:
                            qs_default_console.log(qs_warning_string, repr(e))

            else:
                raise NotImplementedError
            self.save()
        else:
            raise io.UnsupportedOperation

    def save(self):
        """
        保存 | Save

        :return: None
        """
        self.src.close()


class Tar(_NormalCompressedPackage):
    def __init__(self, path, mode="r"):
        """
        Tar协议初始化

        Tar protocol initialization.

        :param path: 压缩包路径 | The package path is compressed.
        :param mode: 工作模式 | Working mode ('read' or 'write')
        """
        super().__init__(tarfile, path, mode)

    def add_file(self, path):
        """
        向压缩包添加文件（工作在'write'模式下）

        Add files to the compressed package (works in 'write' mode)

        :param path: 文件路径
        :return: None
        """
        return super().add_file(path)

    def extract(self):
        """
        解压缩

        extract

        :return: None
        """
        return super().extract()

    def save(self):
        """保存 | save"""
        return super().save()


class Zip(_NormalCompressedPackage):
    def __init__(self, path, mode="r"):
        """
        Zip协议初始化

        Zip protocol initialization.

        :param path: 压缩包路径 | The package path is compressed.
        :param mode: 工作模式 | Working mode ('read' or 'write')
        """
        super().__init__(zipfile, path, mode)

    def add_file(self, path):
        """
        向压缩包添加文件（工作在'w'模式下）

        Add files to the compressed package (works in 'write' mode)

        :param path: 文件路径
        :return: None
        """
        return super().add_file(path)

    def extract(self):
        """
        解压缩

        extract

        :return: None
        """
        return super().extract()

    def save(self):
        """保存 | save"""
        return super().save()


class Rar(_NormalCompressedPackage):
    def __init__(self, path, mode="r"):
        """
        Rar协议初始化

        Rar protocol initialization.

        :param path: 压缩包路径 | The package path is compressed.
        :param mode: 工作模式 | Working mode ('read' or 'write')
        """
        super().__init__(rarfile, path, mode)

    def add_file(self, path):
        """
        向压缩包添加文件（工作在'w'模式下）

        Add files to the compressed package (works in 'write' mode)

        :param path: 文件路径
        :return: None
        """
        return super().add_file(path)

    def extract(self):
        """
        解压缩

        extract

        :return: None
        """
        return super().extract()

    def save(self):
        """保存 | save"""
        return super().save()


class SevenZip(_NormalCompressedPackage):
    def __init__(self, path, mode="r"):
        """
        7z协议初始化

        7z protocol initialization.

        :param path: 压缩包路径 | The package path is compressed.
        :param mode: 工作模式 | Working mode ('read' or 'write')
        """
        super().__init__(py7zr, path, mode)

    def add_file(self, path):
        """
        向压缩包添加文件（工作在'w'模式下）

        Add files to the compressed package (works in 'write' mode)

        :param path: 文件路径
        :return: None
        """
        return super().add_file(path)

    def extract(self):
        """
        解压缩

        extract

        :return: None
        """
        return super().extract()

    def save(self):
        """保存 | save"""
        return super().save()



"""
CPU和内存监测, 但是很丑, 只工作在啥他妈都不支持的Windows命令行中

CPU and memory monitoring, but very ugly, only works in the Windows command line which does not support fuck anything
"""


def top():
    """
    CPU和内存监测

    CPU and memory monitoring

    :return: None
    """
    import time
    import math
    import colorama
    from colorama import Style, ansi, Cursor
    from .. import cur_time, dir_char, requirePackage
    from ..TuiTools import Bar
    from ..NetTools.NormalDL import size_format

    psutil = requirePackage("psutil")
    PrettyTable = requirePackage("prettytable", "PrettyTable")

    def deal():
        print(ansi.clear_screen() + Cursor.POS(0, 0) + Style.RESET_ALL, end="")
        exit(0)

    colorama.init()
    _kernal = psutil.cpu_count()
    _total_mem = psutil.virtual_memory().total
    _cpu_dt = [0] * 40
    _mem_dt = [0] * 40
    _cpu_chart = Bar.RollBar(_cpu_dt, height=10)
    _mem_chart = Bar.RollBar(_mem_dt, height=10)
    charts = [_cpu_chart, _mem_chart]
    window = PrettyTable()
    window.add_row(charts)
    print(ansi.clear_screen())
    try:
        while True:
            _cpu_cur = sum(psutil.cpu_percent(percpu=True)) / _kernal
            _mem_cur = psutil.virtual_memory().used
            _cpu_chart.add(math.ceil(_cpu_cur))
            _mem_chart.add(math.ceil(_mem_cur / _total_mem * 100))
            window.field_names = [
                "CPU: %.2f%%" % _cpu_cur,
                "MEM: %s" % size_format(_mem_cur),
            ]
            print((ansi.clear_screen() if dir_char == "\\" else "") + Cursor.POS(0, 0))
            print(" " * 39, end="")
            cur_time()
            cur_img = "    ".join(str(window).split("\n") + [" "])
            print(cur_img)
            time.sleep(1)
    except:
        deal()



# coding=utf-8
"""
调用各种qs的API

Call various QS API
"""
import sys

from . import (
    user_lang,
    system,
    qs_default_console,
    qs_error_string,
    qs_info_string,
    qs_warning_string,
    requirePackage,
)


def remove_bg():
    """
    删除图片背景

    Delete image background
    """
    try:
        path = sys.argv[2]
    except IndexError:
        qs_default_console.log(
            qs_error_string,
            "%s: qs rmbg <%s>"
            % (("Usage", "picture") if user_lang != "zh" else ("用法", "图像")),
        )
        return
    else:
        if path == "-helpF":
            qs_default_console.print(
                qs_info_string,
                "%s: qs rmbg <%s>"
                % (("Usage", "picture") if user_lang != "zh" else ("用法", "图像")),
            )
            return
        from .API.SimpleAPI import rmbg

        with qs_default_console.status(
            "Dealing.." if user_lang != "zh" else "处理中.."
        ) as st:
            rmbg(path)
            st.update(status="Done")


def smms():
    """
    上传图片或Markdown中图片到smms

    Upload images or Markdown images to SMMS
    """
    try:
        path = sys.argv[2]
    except IndexError:
        qs_default_console.log(
            qs_error_string,
            "%s: qs smms <%s>"
            % (
                ("Usage", "picture | *.md")
                if user_lang != "zh"
                else ("用法", "图像 | *.md")
            ),
        )
        return
    else:
        if path == "-help":
            qs_default_console.print(
                qs_info_string,
                "%s: qs smms <%s>"
                % (
                    ("Usage", "picture | *.md")
                    if user_lang != "zh"
                    else ("用法", "图像 | *.md")
                ),
            )
            return
        from .API.SimpleAPI import smms

        with qs_default_console.status(
            "Dealing.." if user_lang != "zh" else "处理中.."
        ) as st:
            smms(path)
            st.update(status="Done")


def ali_oss():
    """
    阿里云对象存储

    Ali Cloud object storage
    """
    try:
        op = sys.argv[2]
        if op not in ["-dl", "-up", "-ls", "-rm"]:
            raise IndexError
        if op != "-ls" and "--bucket" in sys.argv:
            bucket = sys.argv[sys.argv.index("--bucket") + 1]
        elif op == "-ls":
            bucket = None if len(sys.argv) < 4 else sys.argv[3]
        else:
            bucket = None
        files = sys.argv[3:] if op != "-ls" else []
    except IndexError:
        qs_default_console.print(
            qs_info_string,
            "qs alioss:\n"
            "    -up <file> [bucket]: upload file to bucket\n"
            "    -dl <file> [bucket]: download file from bucket\n"
            "    -rm <file> [bucket]: remove file in bucket\n"
            "    -ls [bucket]       : get file info of bucket",
        ) if user_lang != "zh" else qs_default_console.print(
            qs_info_string,
            "qs alioss:\n"
            "    -up <文件> [桶]: 上传文件至桶\n"
            "    -dl <文件> [桶]: 从桶下载文件\n"
            "    -rm <文件> [桶]: 从桶删除文件\n"
            "    -ls [桶]       : 获取桶文件信息",
        )
        return
    else:
        from .API.AliCloud import AliyunOSS

        ali_api = AliyunOSS()
        func_table = ali_api.get_func_table()
        for file in files:
            func_table[op](file, bucket)
        if not files:
            func_table[op](bucket)


def qiniu():
    """
    七牛云对象存储

    Qiniu cloud object storage
    """
    try:
        op = sys.argv[2]
        if op not in ["-dl", "-up", "-ls", "-rm"]:
            raise IndexError
        if op != "-ls" and "--bucket" in sys.argv:
            bucket = sys.argv[sys.argv.index("--bucket") + 1]
        elif op == "-ls":
            bucket = None if len(sys.argv) < 4 else sys.argv[3]
        else:
            bucket = None
        files = sys.argv[3:] if op != "-ls" else []
    except IndexError:
        qs_default_console.print(
            qs_info_string,
            "qs qiniu:\n"
            "    -up <file> [bucket]: upload file to bucket\n"
            "    -dl <file> [bucket]: download file from bucket\n"
            "    -cp <url > [bucket]: copy file from url\n"
            "    -rm <file> [bucket]: remove file in bucket\n"
            "    -ls [bucket]       : get file info of bucket",
        ) if user_lang != "zh" else qs_default_console.print(
            qs_info_string,
            "qs qiniu:\n"
            "    -up <文件> [桶]: 上传文件至桶\n"
            "    -dl <文件> [桶]: 从桶下载文件\n"
            "    -cp <链接> [桶]: 从链接下载文件到桶\n"
            "    -rm <文件> [桶]: 从桶删除文件\n"
            "    -ls [桶]       : 获取桶文件信息",
        )
        return
    else:
        from .API.QiniuOSS import QiniuOSS

        qiniu_api = QiniuOSS()
        func_table = qiniu_api.get_func_table()
        for file in files:
            func_table[op](file, bucket)
        if not files:
            func_table[op](bucket)


def txcos():
    """
    腾讯云对象存储

    Tencent Cloud object storage
    """
    try:
        op = sys.argv[2]
        if op not in ["-dl", "-up", "-ls", "-rm"]:
            raise IndexError
        if op != "-ls" and "--bucket" in sys.argv:
            bucket = sys.argv[sys.argv.index("--bucket") + 1]
        elif op == "-ls":
            bucket = None if len(sys.argv) < 4 else sys.argv[3]
        else:
            bucket = None
        files = sys.argv[3:] if op != "-ls" else []
    except IndexError:
        qs_default_console.print(
            qs_info_string,
            "qs alioss:\n"
            "    -up <file> [bucket]: upload file to bucket\n"
            "    -dl <file> [bucket]: download file from bucket\n"
            "    -rm <file> [bucket]: remove file in bucket\n"
            "    -ls [bucket]       : get file info of bucket",
        ) if user_lang != "zh" else qs_default_console.print(
            qs_info_string,
            "qs alioss:\n"
            "    -up <文件> [桶]: 上传文件至桶\n"
            "    -dl <文件> [桶]: 从桶下载文件\n"
            "    -rm <文件> [桶]: 从桶删除文件\n"
            "    -ls [桶]       : 获取桶文件信息",
        )
        return
    else:
        from .API.TencentCloud import TxCOS

        tx_api = TxCOS()
        func_table = tx_api.get_func_table()
        for file in files:
            func_table[op](file, bucket)
        if not files:
            func_table[op](bucket)


def translate(content: str = None):
    """
    qs默认的翻译引擎

    Qs default Translation engine

    :param content: 需要翻译的内容
    """
    from . import trans_engine

    pyperclip = requirePackage("pyperclip")
    if trans_engine != "default":
        from .API.TencentCloud import translate
    else:
        from .API.alapi import translate

    output_flag = False if content else True
    if not content:
        content = " ".join(sys.argv[2:])
    if not content:
        try:
            content = pyperclip.paste()
        except:
            from . import qs_default_input

            content = qs_default_input.ask(
                "Sorry, but your system is not supported by `pyperclip`\nSo you need input content manually: "
                if user_lang != "zh"
                else "抱歉，但是“pyperclip”不支持你的系统\n，所以你需要手动输入内容:"
            )
    if content:
        ret = translate(content.replace("\n", " "))
        if output_flag:
            qs_default_console.print(ret) if ret else qs_default_console.log(
                qs_error_string, "Translate Failed!"
            )
        return ret
    else:
        qs_default_console.log(
            qs_warning_string,
            "No content in your clipboard or command parameters!"
            if user_lang != "zh"
            else "剪贴板或命令参数没有内容!",
        )
        return None


def weather():
    """查天气 | Check weather"""
    from . import headers, dir_char
    from .ThreadTools import ThreadFunctionWrapper
    import requests

    def get_data(url):
        try:
            ct = requests.get(url, headers)
        except:
            return
        ct.encoding = "utf-8"
        ct = ct.text.split("\n")
        if dir_char == "/":
            res = ct.copy()
        else:
            import re

            for line in range(len(ct)):
                ct[line] = re.sub("\x1b.*?m", "", ct[line])
            res = ct.copy()
        return res

    try:
        loc = sys.argv[2]
    except IndexError:
        loc = ""
    tls = [
        ThreadFunctionWrapper(
            get_data,
            "https://wttr.in/" + (loc if loc else "?lang={}".format(user_lang)),
        ),
        ThreadFunctionWrapper(get_data, "https://v2.wttr.in/" + loc),
    ]
    for i in tls:
        i.start()
    with qs_default_console.status(
        "Requesting.." if user_lang != "zh" else "请求中.."
    ) as st:
        for i in tls:
            i.join()
        st.update(status="Done")
    simple = tls[0].get_res()
    table = tls[1].get_res()
    if simple:
        if not loc:
            if user_lang == "zh":
                from .API.alapi import translate

                enLocation = simple[0].split("：")[-1].strip()
                trans_loaction = translate(enLocation)
                qs_default_console.print(
                    "地区：" + trans_loaction if trans_loaction else enLocation
                )
            else:
                qs_default_console.print("Location" + simple[0][simple[0].index(":") :])
        simple = simple[2:7]
        print("\n".join(simple))
    else:
        qs_default_console.log(
            qs_error_string, "Get data failed." if user_lang != "zh" else "错误: 获取数据失败"
        )
    if table:
        qs_default_console.print(table[3][:-1])
        bottom_line = 7
        try:
            while "╂" not in table[bottom_line]:
                bottom_line += 1
        except IndexError:
            qs_default_console.log(
                qs_error_string,
                "Get Weather Data failed!" if user_lang != "zh" else "获取天气数据失败",
            )
            return
        for i in table[7 : bottom_line + 2]:
            print(i[:-1])
        print(
            "└────────────────────────────────────────────────────────────────────────"
        )
        print("\n".join(table[-3 if not loc else -4 :]))
    else:
        print("Error: Get data failed." if user_lang != "zh" else "错误: 获取数据失败")


def largeImage():
    """
    百度图片效果增强

    Baidu picture effect enhancement
    """
    try:
        path = sys.argv[2]
    except IndexError:
        qs_default_console.log(
            qs_error_string, "%s: qs LG <img>" % "Usage" if user_lang != "zh" else "用法"
        )
        return
    else:
        from .API.BaiduCloud import ImageDeal

        aip_cli = ImageDeal()
        with qs_default_console.status(
            "Dealing.." if user_lang != "zh" else "处理中.."
        ) as st:
            aip_cli.largeImage(path, st)
            st.update(status="Done")


def AipNLP():
    """百度NLP | Baidu NLP"""
    from .API.BaiduCloud import AipNLP

    pyperclip = requirePackage("pyperclip")
    import pyperclip

    pyperclip.paste()
    ct = sys.argv[2:]
    if not ct:
        try:
            ct = [pyperclip.paste()]
        except:
            from . import qs_default_input

            ct = [
                qs_default_input.ask(
                    "Sorry, but your system is not supported by `pyperclip`\nSo you need input content manually: "
                    if user_lang != "zh"
                    else "抱歉，但是“pyperclip”不支持你的系统\n，所以你需要手动输入内容:"
                )
            ]
    NLP = AipNLP()
    for _id, line in enumerate(ct):
        ct[_id] = NLP.get_res(line)
        if _id == 9:
            qs_default_console.print("...")
        elif _id < 9:
            qs_default_console.print(ct[_id])
    try:
        pyperclip.copy("\n".join(ct))
    except:
        pass


def bili_cover():
    """下载Bilibili视频、直播的封面图片（视频链接、视频号均可识别）"""
    from .API.alapi import bili_cover as bc

    pyperclip = requirePackage("pyperclip")

    try:
        url = sys.argv[2]
    except IndexError:
        try:
            url = pyperclip.paste()
        except:
            qs_default_console.log(
                qs_error_string,
                "Sorry, but your system may not be suppported by `pyperclip`"
                if user_lang != "zh"
                else "抱歉，但是“pyperclip”不支持你的系统",
            )
            return
    if not url:
        qs_default_console.log(
            qs_error_string,
            "Usage: qs bcv <url | video code>"
            if user_lang != "zh"
            else "用法: qs bcv <链接 | 视频码>",
        )
        return
    bc(url)


def gbc():
    """查询中国垃圾分类（且仅支持中文查询）"""
    from .API.alapi import garbage_classification

    try:
        with qs_default_console.status(
            "Requesting data.." if user_lang != "zh" else "请求数据中.."
        ) as st:
            res = garbage_classification(sys.argv[2:])
            st.update(status="Done")
        qs_default_console.print(res, justify="center")
    except Exception as e:
        qs_default_console.print(qs_error_string, repr(e))
        qs_default_console.print(
            qs_error_string,
            "Usage: qs gbc <garbage...>" if user_lang != "zh" else "用法: qs gbc <垃圾...>",
        )


def short_video_info(son_call=False):
    """
    获取短视频信息 | Get short video information

    :return:
    """
    from .API.alapi import short_video_info
    from .NetTools import get_fileinfo, size_format

    pyperclip = requirePackage("pyperclip")
    import re

    try:
        url = sys.argv[2]
    except IndexError:
        try:
            url = pyperclip.paste()
            url = re.findall("https?://(?:[-\w.]|%[\da-fA-F]{2}|[/])+", url)[0]
        except:
            qs_default_console.print(
                qs_error_string,
                "Sorry, but your system may not be suppported by `pyperclip`"
                if user_lang != "zh"
                else "抱歉，但是“pyperclip”不支持你的系统",
            )
            return
    if not url:
        qs_default_console.print(
            qs_error_string,
            "Usage: qs svi <url/video code>"
            if not son_call
            else "Usage: qs svd <url/video code>",
        )
        return
    output_prefix = {
        "title": "Title " if user_lang != "zh" else "标题",
        "video": "Video " if user_lang != "zh" else "视频",
        "cover": "Cover " if user_lang != "zh" else "封面",
        "source": "Source" if user_lang != "zh" else "来源",
    }
    with qs_default_console.status("正在获取视频信息") as _:
        status, res = short_video_info(url.strip("/"))
        if not status:
            qs_default_console.print(qs_info_string, res["title"] + ":" + res["source"])
            return status
        showStatus = "-url" in sys.argv
        qs_default_console.print(
            qs_info_string, "[{}] {}".format(output_prefix["title"], res["title"])
        )
        if res["video_url"]:
            sz = (
                int(get_fileinfo(res["video_url"])[-1].headers["content-length"])
                if not son_call
                else -1
            )
            qs_default_console.print(
                qs_info_string,
                "[{}] {}\n{}".format(
                    output_prefix["video"],
                    size_format(sz, True) if sz > 0 else "--",
                    res["video_url"] if showStatus else "",
                ),
            )
        if showStatus:
            sz = int(get_fileinfo(res["cover_url"])[-1].headers["content-length"])
            qs_default_console.print(
                qs_info_string,
                "[{}] {}\n{}".format(
                    output_prefix["cover"],
                    size_format(sz, True),
                    res["cover_url"] if showStatus else "",
                ),
            )
    if system == "darwin":
        from .ImageTools.ImagePreview import image_preview

        image_preview(res["cover_url"], True)
    if "source" in res and res["source"]:
        qs_default_console.print(
            qs_info_string, "[{}] {}".format(output_prefix["source"], res["source"])
        )
    return res


def short_video_dl():
    """
    下载短视频为mp4格式

    Download short video as mp4

    :return:
    """
    from .NetTools.NormalDL import normal_dl
    from .ImageTools.VideoTools import tomp4
    from . import remove

    res = short_video_info(son_call=True)

    if not res:
        qs_default_console.print(
            qs_error_string, "Download failed" if user_lang != "zh" else "下载失败"
        )
        return

    fileName = res["title"] if res["title"] else "UnknownTitle"
    if "pics" in res and res["pics"]:
        from .ImageTools.ImageTools import topng

        for index, url in enumerate(res["pics"]):
            normal_dl(url, set_name=fileName + str(index + 1))
            topng(fileName + str(index + 1))
            remove(fileName + str(index + 1))
    elif res["video_url"]:
        normal_dl(res["video_url"], set_name=fileName)
        tomp4(fileName)
        remove(fileName)


def acg():
    """
    获取随机acg图片链接（可选择下载）

    Get links to random ACG images (download optional)

    :return:
    """
    import random
    from .API.alapi import acg
    from .API.SimpleAPI import acg2

    st = qs_default_console.status(
        "Requesting data.." if user_lang != "zh" else "请求数据中.."
    )
    st.start()
    try:
        status, acg_link, width, height = random.choice([acg, acg2])()
        qs_default_console.print(
            qs_info_string, f"{'链接' if user_lang == 'zh' else 'LINK'}: {acg_link}"
        ) if status else qs_default_console.log(qs_error_string, acg_link)
        if status:
            qs_default_console.print(
                qs_info_string,
                "尺寸:" if user_lang == "zh" else "SIZE:",
                width,
                "×",
                height,
            )
            if "--save" in sys.argv[2:]:
                from . import dir_char
                from .NetTools.NormalDL import normal_dl

                st.stop()
                acg_link = normal_dl(acg_link)  # disable download status for Windows
            from .ImageTools.ImagePreview import image_preview

            st.update(status="Loading image...\n" if user_lang != "zh" else "加载图片中..\n")
            image_preview(
                open(acg_link) if "--save" in sys.argv[2:] else acg_link,
                "--save" not in sys.argv[2:],
                qs_console_status=st,
            )
            return
    except Exception as e:
        qs_default_console.print(qs_error_string, repr(e))
    finally:
        st.stop()


def bingImg():
    """
    获取bing图片链接（可选择下载）

    Get links to bing images (download optional)

    :return:
    """
    from .API.alapi import bingImg

    st = qs_default_console.status(
        "Requesting data.." if user_lang != "zh" else "请求数据中.."
    )
    st.start()

    try:
        status, acg_link, cprt = bingImg()
        qs_default_console.print(
            qs_info_string, f"{'链接' if user_lang == 'zh' else 'LINK'}: {acg_link}"
        ) if status else qs_default_console.log(qs_error_string, acg_link)
        if status:
            qs_default_console.print(
                qs_info_string, "版权:" if user_lang == "zh" else "CPRT:", cprt
            )
            if "--save" in sys.argv[2:]:
                from .NetTools.NormalDL import normal_dl

                st.stop()
                acg_link = normal_dl(acg_link)
            from .ImageTools.ImagePreview import image_preview

            st.update(status="Loading image..." if user_lang != "zh" else "加载图片中..")
            image_preview(
                open(acg_link) if "--save" in sys.argv[2:] else acg_link,
                "--save" not in sys.argv[2:],
                qs_console_status=st,
            )
    except Exception as e:
        qs_default_console.print(qs_error_string, repr(e))
    finally:
        st.stop()


def preview_html_images():
    """
    获取网页中图片链接（可在Mac::iTerm中自动预览）

    Get links to pictures in the web page (automatically previewed in Mac::iTerm)

    :return:
    """
    from .API.SimpleAPI import imgs_in_url

    save_flag = "--save" in sys.argv
    if save_flag:
        sys.argv.remove("--save")
    for url in sys.argv[2:]:
        imgs_in_url(url, save_flag)


def kdCheck():
    """
    查国内快递

    Check domestic express

    :return:
    """
    from .API.alapi import kdCheck

    with qs_default_console.status(
        "Requesting data.." if user_lang != "zh" else "请求数据中.."
    ):
        status, code, msg = kdCheck(sys.argv[2])
        if not status:
            qs_default_console.print(qs_error_string, msg)
            return

    from .TuiTools.Table import qs_default_table
    from . import table_cell
    from rich.text import Text

    width = qs_default_console.width // 4 * 3
    tb = qs_default_table(
        [
            {
                "header": "Time" if user_lang != "zh" else "时间",
                "justify": "center",
                "style": "bold cyan",
            },
            {
                "header": "Description" if user_lang != "zh" else "描述",
                "justify": "center",
                "no_wrap": False,
            },
            {"header": "Status" if user_lang != "zh" else "状态", "justify": "center"},
        ],
        (
            [
                "[bold underline red]Unknown:heavy_exclamation_mark:",
                "[bold underline yellow]In transit:airplane:",
                "[bold underline green]In delivery:delivery_truck:",
                "[bold underline bold green]Signed receipt:hearts:",
            ][code]
            if user_lang != "zh"
            else [
                "[bold underline red]未知:heavy_exclamation_mark:",
                "[bold underline yellow]运输中:airplane:",
                "[bold underline green]派送中:delivery_truck:",
                "[bold underline magenta]已签收:hearts:",
            ][code]
        )
        + "\n",
    )
    for info in msg[:-1] if code != 3 else msg:
        tb.add_row(
            info["time"],
            Text(table_cell(info["content"], width), justify="full"),
            "[green]:heavy_check_mark:",
        )
    if code != 3:
        tb.add_row(
            msg[-1]["time"],
            Text(table_cell(msg[-1]["content"], width), justify="full"),
            "[bold yellow]:arrow_left:",
        )
    qs_default_console.print(tb, justify="center")


def loli():
    """
    获取一张"可爱"萝莉图的URL，Mac+iTerm2下可在终端预览

    Get the URL of a "cute" Lori map, which can be previewed on the terminal under MAC + iterm2

    :return:
    """
    from .API.Lolicon import loli_img
    from .ImageTools import ImagePreview
    from .NetTools import NormalDL

    with qs_default_console.status(
        "Requesting data.." if user_lang != "zh" else "请求数据中.."
    ):
        status, msg, data = loli_img()
        if not status:
            qs_default_console.print(qs_error_string, msg)
            return

    save_flag = "--save" in sys.argv
    proxy = ""

    if "-p" in sys.argv:
        from . import qs_config

        proxy = qs_config.basicSelect("default_proxy")

    for img in data:
        qs_default_console.print(
            f'[bold underline]{img["title"]} [dim]{img["author"]}', justify="center"
        )
        qs_default_console.print(
            qs_info_string,
            "[bold]link" if user_lang != "zh" else "[bold]链接",
            img["url"],
        )
        qs_default_console.print(
            qs_info_string,
            "[bold]size" if user_lang != "zh" else "[bold]尺寸",
            img["width"],
            "x",
            img["height"],
        )
        if save_flag:
            img["url"] = NormalDL.normal_dl(
                img["url"], set_proxy=proxy, set_referer="https://i.pximg.net"
            )
        if system == "darwin":
            ImagePreview.image_preview(
                open(img["url"]) if save_flag else img["url"],
                not save_flag,
                set_proxy=proxy,
                set_referer="https://i.pximg.net",
            )

        qs_default_console.print(
            "-" * (qs_default_console.width // 4 * 3), justify="center"
        )


def pinyin():
    from .API.alapi import pinyin

    content = " ".join(sys.argv[2:])
    if not content:
        try:
            pyperclip = requirePackage("pyperclip")
            content = pyperclip.paste()
        except:
            from . import qs_default_input

            content = qs_default_input.ask(
                "Sorry, but your system is not supported by `pyperclip`\nSo you need input content manually: "
                if user_lang != "zh"
                else "抱歉，但是“pyperclip”不支持你的系统\n，所以你需要手动输入内容:"
            )
    with qs_default_console.status(
        "Requesting data.." if user_lang != "zh" else "请求数据中.."
    ) as st:
        status, res = pinyin(content)
        st.update(status="Done" if user_lang != "zh" else "请求完成")
    qs_default_console.print(qs_info_string, content)
    qs_default_console.print(qs_info_string if status else qs_error_string, res)


def setu():
    import random

    random.choice([acg, loli])()


def exchange():
    from .API.alapi import exchange

    st = qs_default_console.status(
        "Requesting data.." if user_lang != "zh" else "请求数据中.."
    )
    st.start()
    try:
        status, data = exchange(sys.argv[3], 1)
        qs_default_console.print(
            f"{sys.argv[2]} {sys.argv[3]} ==> {data['exchange']} × {sys.argv[2]} = "
            f"{data['exchange'] * float(sys.argv[2])} {data['currency_to']}\n"
            f"{'Update' if user_lang != 'zh' else '更新时间'}: {data['update_time']}",
            justify="center",
        ) if status else qs_default_console.log(qs_error_string, data)
    except Exception as e:
        qs_default_console.print(qs_error_string, repr(e))
    finally:
        st.stop()


def zhihuDaily():
    from .API.alapi import zhihuDaily
    from rich.panel import Panel
    from .ImageTools.ImagePreview import image_preview

    st = qs_default_console.status(
        "Requesting data.." if user_lang != "zh" else "请求数据中.."
    )
    st.start()
    try:
        status, data = zhihuDaily()
        st.stop()
        if not status:
            qs_default_console.log(qs_error_string, data)
            return
        data = data["stories"] + data["top_stories"]
        for item in data:
            res = "[bold cyan]" + ("Author: " if user_lang != "zh" else "作者: ")
            res += "[bold white]" + item["hint"] + "[/bold white]\n"
            res += "[bold cyan]" + ("Link:" if user_lang != "zh" else "链接: ")
            res += "[bold blue]" + item["url"] + "[/bold blue]"
            if "images" in item:
                if system == "darwin":
                    image_preview(item["images"][0], True)
                else:
                    res += "\n[bold cyan]" + (
                        "Image: \n" if user_lang != "zh" else "图像: \n"
                    )
                    res += "[bold blue]  " + "\n  ".join(item["images"])
            elif "image" in item:
                image_preview(item["image"], True)
            qs_default_console.print(
                Panel(res, title="[b]" + item["title"], width=qs_default_console.width),
                justify="center",
                end="/n/n",
            )
    except Exception as e:
        qs_default_console.print(qs_error_string, repr(e))
    finally:
        st.stop()


def wallhaven():
    from .API.SimpleAPI import wallhaven

    url, oneFlag = "", False
    if "-h" in sys.argv:
        return qs_default_console.print(
            qs_info_string, "Usage: qs wallhaven [--url <url>] [-one] [--save]"
        )

    if "--url" in sys.argv:
        url = sys.argv[sys.argv.index("--url") + 1]
        sys.argv.remove("--url")
        sys.argv.remove(url)

    oneFlag = "-one" in sys.argv

    res = wallhaven(randomOne=oneFlag) if not url else wallhaven(url, randomOne=oneFlag)
    if not res:
        return

    if "--save" in sys.argv:
        from .NetTools.NormalDL import normal_dl
        import os

        for i in res:
            name = i["url"].split("/")[-1]
            if os.path.exists(name):
                qs_default_console.print(
                    qs_warning_string,
                    f"{name} is exists!" if user_lang != "zh" else f"{name} 已存在!",
                )
                continue
            qs_default_console.print(qs_info_string, f"Deal:\t{name}")
            qs_default_console.print(
                qs_info_string, f'Size:\t{" × ".join([str(j) for j in i["size"]])}'
            )
            normal_dl(i["url"], output_error=True, failed2exit=True)
            qs_default_console.print("-" * qs_default_console.width)
    else:
        if oneFlag:
            res = res[0]
            qs_default_console.print(qs_info_string, "URL:\t", res["url"])
            qs_default_console.print(
                qs_info_string, "SIZE:\t", " × ".join([str(i) for i in res["size"]])
            )
            from .ImageTools.ImagePreview import image_preview

            with qs_default_console.status(
                "Getting.." if user_lang != "zh" else "获取图片中.."
            ) as st:
                image_preview(res["url"], True, qs_console_status=st)
        else:
            from .TuiTools.Table import qs_default_table

            tb = qs_default_table(
                [
                    {
                        "header": "ID" if user_lang != "zh" else "编号",
                        "justify": "center",
                        "style": "bold",
                    },
                    {"header": "URL", "justify": "center", "style": "bold cyan"},
                    {
                        "header": "Size" if user_lang != "zh" else "尺寸",
                        "justify": "center",
                    },
                ],
                "Wallhaven Anime TopList\n",
            )
            for iid, item in enumerate(res):
                tb.add_row(
                    str(iid), item["url"], " × ".join([str(i) for i in item["size"]])
                )
            qs_default_console.print(tb, justify="center")


def lmgtfy():
    from .API.SimpleAPI import lmgtfy

    pyperclip = requirePackage("pyperclip")

    keyword = " ".join(sys.argv[2:])
    res = lmgtfy(keyword)
    try:
        pyperclip.copy(res)
        qs_default_console.print(
            qs_info_string,
            "The generated link has been copied for you, and teach your friends how to use Google! (Can also Copy "
            "Following link"
            if user_lang != "zh"
            else "已经为您复制生成的链接，快教朋友如何使用Google吧! (也可以复制下面的链接)",
        )
    except:
        qs_default_console.print(
            qs_warning_string,
            "Failed to use clipboard, you can copy the flowing link:"
            if user_lang != "zh"
            else "调用剪切板失败，你可以复制下面的链接:",
        )
    qs_default_console.print(qs_info_string, res)


def daily60s():
    from .API.alapi import daily60s

    status, res = daily60s()
    if not status:
        return qs_default_console.print(qs_error_string, res)

    from rich.padding import Padding
    import signal
    import time

    cur_index = 0

    with qs_default_console.status("showing" if user_lang != "zh" else "展示中") as st:

        def dealSignal(*argv):
            for index in range(cur_index + 1, len(res["news"])):
                qs_default_console.print(
                    Padding(
                        res["news"][index], (0, int(0.2 * qs_default_console.width))
                    )
                )
                qs_default_console.print()
            qs_default_console.print("-" * qs_default_console.width)
            qs_default_console.print(res["weiyu"] + "\n", justify="center")

            if "--save" in sys.argv:
                import os
                from .NetTools.NormalDL import normal_dl

                st.stop()
                name = normal_dl(res["image"], set_name="news.png")
                st.start()
                qs_default_console.print(
                    "Preview" if user_lang != "zh" else "预览", justify="center"
                )
                from .ImageTools.ImagePreview import image_preview

                st.update(status="loading image.." if user_lang != "zh" else "加载图片中..")
                image_preview(open(name), qs_console_status=st)
            exit(0)

        signal.signal(signal.SIGINT, dealSignal)
        qs_default_console.print(
            "[bold magenta underline]每天60秒读懂世界[/bold magenta underline]",
            justify="center",
        )
        qs_default_console.print(res["date"], justify="center")
        qs_default_console.print("-" * qs_default_console.width)
        len_ls = [len(i) for i in res["news"]]
        total_len = sum(len_ls)
        len_ls = [i / total_len * 60 for i in len_ls]
        len_ls[-1] = 0

        for index, item in enumerate(res["news"]):
            qs_default_console.print(
                Padding(item, (0, int(0.2 * qs_default_console.width)))
            )
            qs_default_console.print()
            cur_index = index
            time.sleep(len_ls[index])
    dealSignal()


def m2t():
    """
    磁力链接转种子文件

    Magnet link to Torrent file
    :return:
    """
    import re

    if "-f" in sys.argv:
        file_path = sys.argv[sys.argv.index("-f") + 1]
        with open(file_path, "r") as f:
            contents = f.read()
    elif "-u" in sys.argv:
        contents = sys.argv[sys.argv.index("-u") + 1]
    else:
        contents = requirePackage("pyperclip", "paste")()
    urls = re.findall("magnet:\?xt=urn:btih:(.*)", contents)
    if len(urls) > 1:
        from .__config__ import prompt

        url = prompt(
            {
                "type": "list",
                "name": "hash",
                "message": "Select hash code | 选择哈希码:",
                "choices": urls,
            }
        )["hash"]
    else:
        url = urls[0]
    from .API.Lolicon import magnet2torrent

    magnet2torrent(url)


def d2m():
    """
    番号搜索并复制磁力链

    :return:
    """
    global url
    try:
        designation = sys.argv[2]
    except IndexError:
        return qs_default_console.print(qs_error_string, "qs d2m <designation>")

    from .__config__ import prompt
    from .API.SimpleAPI import Designation2magnet

    copy = requirePackage("pyperclip", "copy", not_ask=True)
    PyperclipException = requirePackage("pyperclip", "PyperclipException")
    copied = False

    searcher = Designation2magnet(designation)
    infos = searcher.search_designation()

    choices = [f"[{n + 1}] " + i[1] + ": " + i[-1] for n, i in enumerate(infos)]
    try:
        url = searcher.get_magnet(
            infos[
                choices.index(
                    prompt(
                        {
                            "type": "list",
                            "message": "Select | 选择",
                            "name": "sub-url",
                            "choices": choices,
                        }
                    )["sub-url"]
                )
            ][0]
        )
        if copy:
            copy(url)
            copied = True
        else:
            raise PyperclipException
    except KeyError:
        return
    except PyperclipException:
        qs_default_console.print(qs_info_string, url)

    if copied:
        qs_default_console.print(
            qs_info_string,
            "magnet url copied to clipboard" if user_lang != "zh" else "磁力链接已拷贝至粘贴板",
        )


def doutu():
    """
    获取关键词的随机十张表情包
    :return:
    """
    try:
        keyword = sys.argv[2]
    except IndexError:
        return qs_default_console.print(qs_error_string, "qs doutu <designation>")

    from .API.alapi import doutu
    from .ImageTools.ImagePreview import image_preview

    status, urls = doutu(keyword)
    if not status:
        qs_default_console.print(qs_error_string, urls)
    else:
        import random

        urls = random.choices(urls, k=10)

        for item in urls:
            qs_default_console.print(qs_info_string, item)
            image_preview(item, is_url=True)


def joke():
    """
    获取段子
    :return:
    """
    import re
    from .API.alapi import joke
    from rich.panel import Panel

    status, ct = joke()
    if not status:
        qs_default_console.print(qs_error_string, ct)
        return
    content = re.sub("&(.*?);", "", ct["content"])
    qs_default_console.print(
        Panel(
            content,
            title="[b magenta]" + ct["title"],
            title_align="center",
            width=qs_default_console.width,
            subtitle=ct["time"],
        )
    )


def gpt():
    """
    Chat GPT-3
    :return:
    """
    from .API.ChatGPT import chatGPT
    from . import _ask, table_cell

    qs_default_console.print(qs_info_string, "Type 'exit' to exit" if user_lang != "zh" else "输入 'exit' 退出")

    while (
        prompt := _ask(
            {
                "type": "input",
                "message": "Input Question" if user_lang != "zh" else "输入问题",
            }
        )
    ) != "exit":
        with qs_default_console.status(
            "Thinking..." if user_lang != "zh" else "思考中..."
        ):
            res = chatGPT(prompt)
        res = "\n".join(
            [table_cell(i, qs_default_console.width) for i in res.split("\n")]
        )
        qs_default_console.print(
            "[bold green]" + ("Answer" if user_lang != "zh" else "回答") + "[/]\n",
            justify="center",
        )
        qs_default_console.print(res)



# coding=utf-8
"""
qs的网络工具库

The network tool library of QS
"""
import socket
import requests
from requests.exceptions import RequestException
from typing import Tuple


headers = {
    "User-Agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_13_2) AppleWebKit/604.4.7 (KHTML, like Gecko) "
    "Version/11.0.2 Safari/604.4.7"
}


def is_ipv4(ip: str) -> bool:
    """
    判断ip是否为可连接的ipv4

    Determine whether the IP is a connectable IPv4
    :param ip: like 8.8.8.8 etc.
    :return: bool
    """
    try:
        socket.inet_pton(socket.AF_INET, ip)
    except AttributeError:  # no inet_pton here, sorry
        try:
            socket.inet_aton(ip)
        except socket.error:
            return False
        return ip.count(".") == 3
    except socket.error:  # not a valid ip
        return False
    return True


def is_ipv6(ip: str) -> bool:
    """
    判断ip是否为可连接的ipv6

    Determine whether the IP is a connectable IPv6
    :param ip: like fe80::1862:5a79:a8a0:aae5 etc.
    :return: bool
    """
    try:
        socket.inet_pton(socket.AF_INET6, ip)
    except socket.error:  # not a valid ip
        return False
    return True


def is_ip(ip: str) -> bool:
    """
    判断ip是否为可连接的

    Determine whether the IP is a connectable
    :param ip: like fe80::1862:5a79:a8a0:aae5, 8.8.8.8 etc.
    :return: bool
    """
    if ip == "localhost":
        return True
    return is_ipv4(ip) or is_ipv6(ip)


def is_mac(addr: str) -> bool:
    """
    检查addr是否为mac地址

    Check whether addr is a mac address

    :param addr: string like '80:8f:1d:e2:f2:f5'
    :return: bool
    """
    part = addr.split(":")
    if len(part) != 6:
        return False
    for i in part:
        if len(i) != 2:
            return False
        for j in i:
            if j not in "0123456789abcdefABCDEF":
                return False
    return True


def check_one_page(url: str) -> bool:
    """
    检查url是否可访问

    Check that the URL is accessible

    :param url: url
    :return: True或False
    """
    try:
        response = requests.head(url, headers=headers).status_code
        return response == requests.codes.ok
    except RequestException:
        return False


def formatUrl(try_url: str) -> str:
    """
    为url添加https或http使其能被访问

    Add HTTPS or HTTP to the URL to make it accessible

    :param try_url: 待尝试的url
    :return: 能被成功访问的url
    """
    if try_url.startswith("http://") or try_url.startswith("https://"):
        return try_url
    res_url = try_url
    if not check_one_page(res_url):
        res_url = "https://" + try_url
        if not check_one_page(res_url):
            res_url = "http://" + try_url
    return res_url


def open_url(url: str):
    """
    使用默认浏览器打开url

    Open url using the default browser

    :param url:
    :return:
    """
    import webbrowser as wb

    url = formatUrl(url)
    wb.open_new_tab(url)


def get_ip() -> str:
    """
    获取本机ip

    Get native IP

    :return: ip
    """
    import socket

    s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    try:
        s.connect(("8.8.8.8", 80))
        ip = s.getsockname()[0]
        return ip
    except Exception:
        s.close()
        return socket.gethostbyname(socket.gethostname())


def size_format(sz: int, align: bool = False) -> str:
    """
    格式化文件大小显示

    Format file size display

    :param sz: 整数，表示文件大小
    :param align: 是否对齐
    :return: 文件大小字符串
    """
    if sz >= 1e9:
        return "%.3f GB" % (sz / 1e9) if not align else "%7.3f GB" % (sz / 1e9)
    elif sz >= 1e6:
        return "%.3f MB" % (sz / 1e6) if not align else "%7.3f MB" % (sz / 1e6)
    elif sz >= 1e3:
        return "%.3f KB" % (sz / 1e3) if not align else "%7.3f KB" % (sz / 1e3)
    else:
        return "%.2f B" % sz if not align else "%7.2f B" % sz


def get_ip_info() -> dict:
    """
    通过ip-api获取本机ip信息

    Get native IP information through IP-API

    :return: ip信息的dict，失败返回None
    """
    import json

    res = requests.get("http://ip-api.com/json/", headers=headers)
    if res.status_code == requests.codes.ok:
        return json.loads(res.text)
    else:
        return {}


def get_fileinfo(
    url: str, proxy: str = "", referer: str = ""
) -> Tuple[str, str, requests.Response]:
    """
    获取待下载的文件信息

    Gets information about the file to be downloaded

    :param url: 文件url
    :param proxy: 代理
    :param referer: 绕反爬
    :return: 真实url，文件名，http头部信息 (headers中键值均为小写)
    """
    import re
    import os

    proxies = {"http": "http://" + proxy, "https": "https://" + proxy} if proxy else {}
    if referer:
        headers["referer"] = referer
    try:
        res = requests.head(url, headers=headers, proxies=proxies)
    except Exception as e:
        return "", repr(e), None
    while res.status_code in [301, 302]:
        url = {i[0]: i[1] for i in res.headers.lower_items()}["location"]
        res = requests.head(url, headers=headers, proxies=proxies)
    res.headers = {i[0]: i[1] for i in res.headers.lower_items()}
    if "content-disposition" in res.headers:
        try:
            filename = re.findall(
                "filename=(.*?);", res.headers["content-disposition"]
            )[0]
        except IndexError:
            from urllib.parse import urlparse

            filename = os.path.basename(urlparse(url).path.strip("/"))
    else:
        from urllib.parse import urlparse

        filename = os.path.basename(urlparse(url).path.strip("/"))
    return url, re.sub(r"^\W+|\W+$", "", filename), res



from .. import requirePackage
from . import pre_check

_openai = None


def chatGPT(
    prompt: str,
):
    """
    使用OpenAI的GPT-3 API进行聊天
    """
    global _openai
    if _openai is None:
        _openai = requirePackage("openai")
        _openai.api_key = pre_check("openai")
    response = _openai.Completion.create(
        engine="text-davinci-003",
        prompt=prompt,
        max_tokens=1024,
        n=1,
        temperature=0.5,
    )
    return response["choices"][0]["text"].strip()



# coding=utf-8
"""
Docs for different language:
    中文: https://rhythmlian.cn/2020/08/09/QuickStart-Rhy-zh/

    English: https://rhythmlian.cn/2020/02/14/QuickStart-Rhy/
"""
import os
import sys

from rich.prompt import Prompt as qs_default_input

from .NetTools import headers
from .__cache__ import QsCache
from .__config__ import QsConfig, dir_char, system, qs_default_console, prompt

name = "QuickStart_Rhy"

user_root = os.path.expanduser("~") + dir_char
qs_config = QsConfig(user_root + ".qsrc", os.path.exists(user_root + ".qsrc"))
qs_cache = QsCache(user_root + ".qs_cache")

user_lang = qs_config.basicSelect("default_language")
user_currency = qs_config.basicSelect("default_currency")
trans_engine = qs_config.basicSelect("default_translate_engine")["support"][
    qs_config.basicSelect("default_translate_engine")["index"]
]
user_pip = qs_config.basicSelect("default_pip")
force_show_img = qs_config.basicSelect("force_show_img")

qs_error_string = f'[bold red][{"ERROR" if user_lang != "zh" else "错误"}]'
qs_warning_string = f'[bold yellow][{"WARNING" if user_lang != "zh" else "警告"}]'
qs_info_string = f'[bold cyan][{"INFO" if user_lang != "zh" else "提示"}]'
qs_console_width = qs_default_console.width


def external_exec(
    cmd: str,
    without_output: bool = False,
    without_stdout: bool = False,
    without_stderr: bool = False,
):
    """
    外部执行命令

    :param cmd: 命令
    :param without_output: 是否不输出
    :param without_stdout: 是否不输出stdout
    :param without_stderr: 是否不输出stderr
    :return: status code, output
    """
    # import threading
    from subprocess import Popen, PIPE
    from concurrent.futures import ThreadPoolExecutor, wait

    class MixContent:
        def __init__(self):
            self.content = ""

        def __add__(self, other):
            self.content += other
            return self

        def __str__(self):
            return self.content

    content = MixContent()

    def _output(pipe_name: str, process: Popen, content: MixContent):
        ignore_status = (
            without_stdout if pipe_name == "stdout" else without_stderr
        ) or without_output
        for line in iter(eval(f"process.{pipe_name}.readline"), ""):
            if not ignore_status:
                qs_default_console.print(line.strip())
            content += line

    pool = ThreadPoolExecutor(2)
    p = Popen(cmd, shell=True, stdout=PIPE, stderr=PIPE, bufsize=1, encoding="utf-8")

    wait(
        [
            pool.submit(_output, "stdout", p, content),
            pool.submit(_output, "stderr", p, content),
        ]
    )
    pool.shutdown()
    ret_code = p.wait()

    return ret_code, str(content)


def requirePackage(
    pname: str,
    module: str = "",
    real_name: str = "",
    not_exit: bool = True,
    not_ask: bool = False,
    set_pip: str = user_pip,
):
    """
    获取本机上的python第三方库，如没有则询问安装

    :param not_ask: 不询问，无依赖项则报错
    :param set_pip: 设置pip路径
    :param pname: 库名
    :param module: 待引入的模块名，可缺省
    :param real_name: 用于 pip3 install 的名字
    :param not_exit: 安装后不退出
    :return: 库或模块的地址
    """
    try:
        exec(f"from {pname} import {module}" if module else f"import {pname}")
    except (ModuleNotFoundError, ImportError):
        if not_ask:
            return None
        confirm = prompt(
            {
                "type": "confirm",
                "name": "install",
                "message": f"""Qs require {pname + (' -> ' + module if module else '')}, confirm to install?  
  Qs 依赖 {pname + (' -> ' + module if module else '')}, 是否确认安装?""",
                "default": True,
            }
        )["install"]
        if confirm:
            with qs_default_console.status(
                "Installing..." if user_lang != "zh" else "正在安装..."
            ):
                external_exec(
                    f"{set_pip} install {pname if not real_name else real_name} -U",
                    True,
                )
            if not_exit:
                exec(f"from {pname} import {module}" if module else f"import {pname}")
            else:
                qs_default_console.print(
                    qs_info_string, f'just run again: "{" ".join(sys.argv)}"'
                )
                exit(0)
        else:
            exit(-1)
    finally:
        return eval(f"{module if module else pname}")


def _ask(question: dict, timeout: int = 0):
    if timeout:

        def ask():
            def handle(signum, frame):
                raise RuntimeError

            import signal

            try:
                signal.signal(signal.SIGALRM, handle)
                signal.alarm(timeout)
                res = prompt(question)
                signal.alarm(0)
                return res
            except RuntimeError:
                if dir_char == "/":
                    os.system("stty echo")
                qs_default_console.print(
                    "\n[bold yellow][Warning | 警告][/bold yellow]",
                    f"Time out & Return | 超时并返回: {question['default'] if 'default' in question else None}",
                )
                return question["default"] if "default" in question else None

    else:

        def ask():
            return prompt(question)[question["name"]]

    try:
        if "name" not in question:
            question["name"] = "NoName"
        return ask()
    except:
        exit(0)


def cut_string(string: str, length: int) -> list:
    """
    每隔l个字符切分字符串

    :param string: 字符串
    :param length: 切分长度
    :return: 切分后产生的list
    """
    string = string.strip().replace("\n", " ")
    res, cur, cnt = [], "", 0
    for i in string:
        cnt += 2 if ord(i) > 255 else 1
        if cnt <= length:
            cur += i
        else:
            res.append(cur)
            cur, cnt = i, 2 if ord(i) > 255 else 1
    if cur:
        res.append(cur)
    return res


def _is_in_path(cmd: str):
    """
    判断cmd是否在环境变量中

    :param cmd: 命令
    :return: 是否在环境变量中
    """
    import os

    ls = os.environ["PATH"].split(os.pathsep)
    for prefix in ls:
        if os.path.exists(os.path.join(prefix, cmd)):
            return True
    return False


def table_cell(string: str, length: int) -> str:
    return "\n".join(cut_string(string, length))


def deal_ctrl_c(signum, frame):
    """
    默认的处理ctrl c的函数

    default function to deal [CTRL C]

    :param signum: signum
    :param frame: frame
    :return: None
    """
    if signum or frame or True:
        exit(0)


def remove(path):
    """
    删除文件或文件夹

    delete file or folder.

    :param path: 路径
    :return: None
    """
    if os.path.exists(path):
        if os.path.isdir(path):
            import shutil

            shutil.rmtree(path)
        else:
            os.remove(path)


def cur_time():
    """
    获取当前时间

    get time of now

    :return: None
    """
    week = {
        "Monday": "周一",
        "Tuesday": "周二",
        "Wednesday": "周三",
        "Thursday": "周四",
        "Friday": "周五",
        "Saturday": "周六",
        "Sunday": "周日",
    }
    import time

    tm = time.strftime("%Y年%m月%d日 %A %H:%M:%S", time.localtime(time.time())).split()
    tm[1] = week[tm[1]]
    qs_default_console.print(qs_info_string, " ".join(tm))


def open_url(argv: list = None):
    """
    打开命令行参数中的链接

    open urls in argv

    :return: None
    """
    import webbrowser as wb
    from .NetTools import formatUrl

    if not argv:
        argv = sys.argv[2:]
    if argv:
        for url in argv:
            url = formatUrl(url)
            wb.open_new_tab(url)
    else:
        pyperclip = requirePackage("pyperclip")
        try:
            url = pyperclip.paste()
        except:
            url = qs_default_input.ask(
                "Sorry, but your system is not supported by `pyperclip`\nSo you need input content manually: "
                if user_lang != "zh"
                else "抱歉，但是“pyperclip”不支持你的系统\n，所以你需要手动输入内容:",
                console=qs_default_console,
            )
        wb.open_new_tab(formatUrl(url))


def open_app():
    """
    Mac OS下的open（不支持其他平台）

    open app (only support Mac OS)

    :return: None
    """
    if system == "darwin":
        external_exec('open -a "' + '" "'.join(sys.argv[2:]) + '"')
    else:
        return qs_default_console.print(
            qs_error_string,
            '"copy" is only support Mac OS X'
            if user_lang != "zh"
            else '"copy" 只支持Mac OS X',
        )


def open_file(*argv):
    """
    使用合适应用打开文件

    Open file with appropriate application

    :return: None
    """
    if not argv:
        argv = sys.argv[2:]
    if system == "darwin":
        external_exec('open "' + '" "'.join(argv) + '"')
    elif system == "linux":
        external_exec('xdg-open "' + '" "'.join(argv) + '"')
    else:
        for file in argv:
            os.startfile(os.path.abspath(file))


def calculate():
    """
    执行算数表达式

    Execute the arithmetic expression.

    :return: None
    """
    try:
        exp = " ".join(sys.argv[2:])
        qs_default_console.print("%s = %s" % (exp, eval(exp)))
    except Exception as e:
        qs_default_console.print(qs_info_string, 'Usage: qs cal <exp like "1+1">')
        qs_default_console.print(qs_error_string, repr(e))


def pcat():
    """
    输出粘贴板中内容

    Output the contents of the clipboard
    :return: None
    """
    print(requirePackage("pyperclip", "paste")())


def fcopy():
    """
    获取文件内容并复制进粘贴板

    Get the content of the file and copy it into the pasteboard
    :return:
    """
    if not os.path.exists(sys.argv[2]):
        return qs_default_console.print(
            qs_error_string,
            "No such file:" if user_lang != "zh" else "未找到文件:",
            sys.argv[2],
        )
    with open(sys.argv[2], "r") as f:
        requirePackage("pyperclip").copy(f.read())


def copy():
    """
    复制文件到粘贴板
    :return:
    """

    def which(command):
        return False if external_exec("which %s" % command, True)[0] else True

    if system != "darwin":
        return qs_default_console.print(
            qs_error_string,
            '"copy" is only support Mac OS X'
            if user_lang != "zh"
            else '"copy" 只支持Mac OS X',
        )

    if not os.path.exists(sys.argv[2]):
        return qs_default_console.print(
            qs_error_string,
            "No such file:" if user_lang != "zh" else "未找到文件:",
            sys.argv[2],
        )
    # 检查 pbadd 是否在 PATH 中
    if not which("pbadd"):
        from QuickStart_Rhy.NetTools.NormalDL import normal_dl

        normal_dl(
            "https://cos.rhythmlian.cn/ImgBed/86438ea0f489a2c75ff7263eda630005",
            set_name="pbadd",
        )
        external_exec("chmod +x pbadd")
        external_exec("mv pbadd /usr/local/bin/")
    external_exec("pbadd " + sys.argv[2], True)


def get_user_lang():
    print(user_lang)


def play_music():
    AS = requirePackage("pydub", "AudioSegment")
    play = requirePackage("pydub.playback", "play")

    for music in sys.argv[2:]:
        try:
            play(AS.from_file(music))
        except:
            pass


def qs_print(*argv):
    """
    打印文件

    :return:
    """
    if not argv:
        argv = sys.argv[2:]
    if dir_char != "/":
        win32api = requirePackage("pywin32", "win32api")
        win32print = requirePackage("pywin32", "win32print")
        for file in argv:
            win32api.ShellExecute(
                0, "print", file, '/d:"%s"' % win32print.GetDefaultPrinter(), ".", 0
            )
    else:
        for file in argv:
            external_exec(f"lp {file}")


def sas():
    """
    设置音源

    Set audio source

    :return: None
    """
    if system != "darwin":
        qs_default_console.print(
            qs_error_string,
            "Not support your system" if user_lang != "zh" else "不支持你的系统",
        )
        return
    if not _is_in_path("SwitchAudioSource"):
        if _ask(
            {
                "type": "confirm",
                "message": "Install SwitchAudioSource?"
                if user_lang != "zh"
                else "安装 SwitchAudioSource？",
                "default": True,
            }
        ):
            with qs_default_console.status(
                "Installing SwitchAudioSource..."
                if user_lang != "zh"
                else "正在安装 SwitchAudioSource..."
            ):
                retries = 3
                while retries:
                    external_exec("brew install SwitchAudioSource")
                    if _is_in_path("SwitchAudioSource"):
                        break
                    retries -= 1
                if not retries:
                    qs_default_console.print(
                        qs_error_string,
                        "Install failed" if user_lang != "zh" else "安装失败",
                    )
                    return
    import json

    current = json.loads(external_exec("SwitchAudioSource -c -f json", True)[1])
    devices = [
        json.loads(i)
        for i in external_exec("SwitchAudioSource -a -f json", True)[1].splitlines()
    ]
    devices = {
        i["id"]: {
            "uid": i["uid"],
            "type": "🔈" if i["type"] == "output" else "🎤",
            "name": i["name"],
        }
        for i in devices
    }
    if not devices:
        qs_default_console.print(
            qs_error_string, "No device found" if user_lang != "zh" else "未找到设备"
        )
        return
    from .TuiTools.Table import qs_default_table

    table = qs_default_table(
        ["ID", "Name", "Type", "Current"]
        if user_lang != "zh"
        else ["ID", "名称", "类型", "当前设备"]
    )
    for i in devices:
        table.add_row(i, devices[i]["name"], devices[i]["type"]) if devices[i][
            "uid"
        ] != current["uid"] else table.add_row(
            i, devices[i]["name"], devices[i]["type"], "[bold green]⬅[/]"
        )
    qs_default_console.print(table, justify="center")

    default = qs_cache.get("audio_source")
    uids = {val["uid"]: key for key, val in devices.items()}
    question = {
        "type": "input",
        "message": "Input device ID" if user_lang != "zh" else "输入设备 ID",
        "validate": lambda x: x in [str(i) for i in devices],
    }
    if default:
        _default = default.copy()
        for item in default:
            if item not in uids:
                _default.pop(item)
        if _default:
            if len(_default) > 1:
                _default.pop(current["uid"])
            question["default"] = uids[max(_default, key=_default.get)]
    else:
        default = {}
    select = _ask(question)
    uid = devices[select]["uid"]
    default[uid] = default.get(uid, 0) + 1

    external_exec("SwitchAudioSource -i %s" % select, True)
    qs_cache.set("audio_source", default)
    qs_default_console.print(qs_info_string, "Done" if user_lang != "zh" else "完成")



# coding=utf-8
"""
一些可以轻易实现的API

Some APIs that can be easily implemented
"""
from . import *
from .. import qs_default_console, qs_error_string, qs_info_string
import json
import requests


def rmbg(filePath: str):
    """
    删除图片背景

    Delete image background

    :param filePath: 图片的路径
    :return: None
    """
    api_key = pre_check("rmbg")
    res = requests.post(
        "https://api.remove.bg/v1.0/removebg",
        files={"image_file": open(filePath, "rb")},
        data={"size": "auto"},
        headers={"X-Api-Key": api_key},
    )
    if res.status_code == requests.codes.ok:
        img_name = filePath.split(dir_char)[-1].split(".")[0]
        if dir_char in filePath:
            img_root = dir_char.join(filePath.split(dir_char)[:-1]) + dir_char
        else:
            img_root = ""
        with open(img_root + img_name + "_rmbg.png", "wb") as imgfile:
            imgfile.write(res.content)
    else:
        qs_default_console.log(qs_error_string, res.status_code, res.text)


def smms(filePath: str):
    """
    上传图片或Markdown中所有的图片到smms中

    Upload images or all images from Markdown to SMMS

    :param filePath: 图片或Markdown文件的路径
    :return: None
    """
    import os
    from ..TuiTools.Table import qs_default_table

    api_key = pre_check("smms")
    res_tb = qs_default_table(
        ["File", "Status", "url"] if user_lang != "zh" else ["文件", "状态", "链接"]
    )

    def post_img(path: str) -> dict:
        try:
            data = {"smfile": (path.split("/")[-1], open(path, "rb")), "format": "json"}
        except:
            return {}
        res_json = requests.post(
            "https://sm.ms/api/v2/upload",
            headers={"Authorization": api_key},
            files=data,
        ).text
        return json.loads(res_json)

    def get_path(rt, rel):
        return os.path.abspath(rt + rel)

    def format_markdown(path):
        import re

        _user_path = os.path.expanduser("~")
        rt_path = dir_char.join(os.path.abspath(path).split(dir_char)[:-1]) + dir_char
        img_set = {}
        with open(path, "r") as fp:
            ct = fp.read()
        aims = re.findall("!\[.*?]\((.*?)\)", ct, re.M) + re.findall(
            '<img.*?src="(.*?)".*?>', ct, re.M
        )
        for aim in aims:
            raw_path = aim
            aim = aim.replace("~", _user_path)
            aim = aim if aim.startswith(dir_char) else get_path(rt_path, aim)
            if aim not in img_set:
                res_dict = post_img(aim)
                if not res_dict:
                    res_tb.add_row(aim.split(dir_char)[-1], "No File", "")
                    img_set[aim] = False
                else:
                    res_tb.add_row(
                        aim.split(dir_char)[-1],
                        res_dict["success"],
                        res_dict["message"]
                        if not res_dict["success"]
                        else res_dict["data"]["url"],
                    )
                    if not res_dict["success"] and res_dict["code"] == "unauthorized":
                        break
                    img_set[aim] = (
                        res_dict["data"]["url"] if res_dict["success"] else False
                    )
            if img_set[aim]:
                ct = ct.replace(raw_path, img_set[aim])
        with open(path, "w") as fp:
            fp.write(ct)
        qs_default_console.print(res_tb, justify="center")

    try:
        is_md = filePath.endswith(".md")
    except IndexError:
        exit("Usage: qs {*.md} | {picture}")
    else:
        if is_md:
            format_markdown(filePath)
        else:
            res = post_img(filePath)
            if not res:
                res_tb.add_row(filePath.split(dir_char)[-1], "No File", "")
            else:
                res_tb.add_row(
                    filePath.split(dir_char)[-1],
                    res["success"],
                    "" if not res["success"] else res["data"]["url"],
                )
            qs_default_console.print(res_tb, justify="center")


def imgs_in_url(url: str, save: bool = False):
    """
    提取url中的img标签链接

    Extract img tag links from url

    :param url:
    :param save:
    :return:
    """
    from .. import headers

    html = requests.get(url, headers=headers)
    if html.status_code != requests.codes.ok:
        qs_default_console.log(
            qs_error_string, "Network Error" if user_lang != "zh" else "网络错误"
        )
        return
    import re
    from ..ImageTools.ImagePreview import image_preview

    normal_dl = None
    if save:
        from ..NetTools.NormalDL import normal_dl

    imgs = re.findall('<img.*?src="(.*?)".*?>', html.text)
    a_ls = re.findall('<a.*?href="(.*?)".*?>', html.text)
    for i in a_ls:
        aim = i.lower()
        for j in [".png", ".jpg", "jpeg"]:
            if j in aim:
                imgs.append(i)
                break
    for url in imgs:
        if not url.startswith("http") or url.endswith("svg"):
            continue
        url = url.replace("&#46;", ".")
        qs_default_console.log(
            qs_info_string,
            "Link:" if user_lang != "zh" else "链接:",
            url[:100] + ("" if len(url) <= 100 else "..."),
        )
        if save:
            file_name = normal_dl(url)
        else:
            file_name = url
        image_preview(file_name, True)


def acg2():
    """
    随机获取一张acg图片链接

    Get a random link to an ACG image

    :return: 请求状态, 链接或报错, 宽度, 高度 | status, url or error, width, height
    """
    try:
        res = requests.get("https://api.luvying.com/acgimg?return=json")
    except Exception as e:
        return False, repr(e), None, None
    else:
        import json

        res = json.loads(res.text)
        return (
            res["code"] == "200",
            (res["acgurl"] if res["code"] == "200" else "Error"),
            res["width"],
            res["height"],
        )


def wallhaven(
    set_search_url: str = pre_check("wallhaven_aim_url", False), randomOne: bool = False
):
    """
    获取wallhaven toplist或指定图片列表

    Get wallhaven toplist or specified picture list

    :param set_search_url: 用于搜索的URL
    :param randomOne: 随机抽一张返回
    :return: 包含链接的列表 | [link1, link2, ...]
    """
    from .. import qs_default_console, qs_error_string
    from . import headers
    import requests
    import re

    if not set_search_url:
        set_search_url = "https://wallhaven.cc/search?categories=010&purity=011&topRange=1M&sorting=toplist&order=desc"

    urlTemplate = "https://w.wallhaven.cc/full/{}/wallhaven-{}"
    html = requests.get(set_search_url, headers=headers)
    if html.status_code != requests.codes.ok:
        qs_default_console.print(qs_error_string, f"Http Status: {html.status_code}")
        return None
    html = re.findall("<section.*?>(.*?)</section", html.text, re.S)[0]
    lis = re.findall("<li>(.*?)</li", html, re.S)
    res = []
    for i in lis:
        url, size = re.findall(
            '<img.*?data-src="(.*?)".*?<span.*?class="wall-res".*?>(.*?)<', i, re.S
        )[0]
        url = url.split("/")[-2:]
        if '<span class="png">' in i:
            url[-1] = url[-1].replace(".jpg", ".png")
        res.append(
            {
                "url": urlTemplate.format(*url),
                "size": [int(i) for i in re.findall("\d+\.?\d*", size)],
            }
        )
    if randomOne:
        import random

        return [random.choice(res)]
    return res


def lmgtfy(keyword: str):
    """
    让我帮你Google一下

    Let me google that for you

    :param keyword: 关键词
    :return: 目标链接 | http link
    """
    import random
    import base64

    supportLs = ["https://moedog.org/tools/google/?q="]
    return random.choice(supportLs) + base64.b64encode(bytes(keyword, "utf-8")).decode(
        "utf-8"
    )


class Designation2magnet:
    import re

    def __init__(self, description):
        self.rt_url = "https://18mag.net"
        self.cover_url = "https://www5.javmost.com/search/{designation}/"
        self.description = description

    def search_designation(self):
        """
        番号搜索

        :return:
        """
        infos = [
            Designation2magnet.re.findall(
                '<a.*?href="(.*?)".*?>(.*?)<.*?td-size.*?>(.*?)<',
                item,
                Designation2magnet.re.S,
            )
            for item in Designation2magnet.re.findall(
                "<tr>(.*?)</tr>",
                requests.get(self.rt_url + "/search?q={}".format(self.description))
                .text.replace("<b>", "")
                .replace("</b>", ""),
                Designation2magnet.re.S,
            )
        ]
        if [] in infos:
            return [
                (i[0][0], i[0][1].strip(), i[0][2]) for i in infos[: infos.index([])]
            ]
        else:
            return [(i[0][0], i[0][1].strip(), i[0][2]) for i in infos]

    def get_magnet(self, info: str):
        """
        将选定的信息转为磁力链

        :param info:
        :return:
        """
        return (
            "magnet:?xt=urn:btih:"
            + Designation2magnet.re.findall(
                "种子特征码.*?<dd>(.*?)<",
                requests.get(self.rt_url + info).text,
                Designation2magnet.re.S,
            )[0]
        )



# coding=utf-8
"""
命令交互，查看qs的各种菜单和文档

Command interaction, view various menus and documents of QS
"""
from . import user_lang, open_url
from .__config__ import prompt


def onlineDocuments():
    res = prompt(
        {
            "type": "list",
            "name": "onlineSource",
            "message": "Switch a website source:"
            if user_lang != "zh"
            else "选择合适你的网站源:",
            "choices": [
                "1. CDN in China" if user_lang != "zh" else "1. 国内CDN",
                "2. Github Page (non-Chinese users)"
                if user_lang != "zh"
                else "2. Github Page (非中国用户)",
            ],
        }
    )["onlineSource"]
    import webbrowser as wb

    wb.open(
        {
            "1": "https://rhythmlian.cn/"
            + (
                "2020/02/14/QuickStart-Rhy/"
                if user_lang != "zh"
                else "2020/08/09/QuickStart-Rhy-zh/"
            ),
            "2": "https://rhythmicc.github.io/"
            + (
                "2020/02/14/QuickStart-Rhy/"
                if user_lang != "zh"
                else "2020/08/09/QuickStart-Rhy-zh/"
            ),
        }[res[0]]
    )


def lesson():
    mainMenu = {
        "type": "list",
        "name": "action",
        "message": "Welcome using qs, choose and get help:"
        if user_lang != "zh"
        else "欢迎使用qs, 选择以下选项获得帮助:",
        "choices": [
            "1. basic tools" if user_lang != "zh" else "1. 基础工具",
            "2. system tools" if user_lang != "zh" else "2. 系统工具",
            "3. network tools" if user_lang != "zh" else "3. 网路工具",
            "4. api tools" if user_lang != "zh" else "4. 扩展工具",
            "5. image tools" if user_lang != "zh" else "5. 图像工具",
            "6. online Docs" if user_lang != "zh" else "6. 在线文档",
            "7. join TG group" if user_lang != "zh" else "7. 加入TG群",
        ],
    }
    res = prompt(mainMenu)["action"]

    if res[0] == "7":
        return open_url(["https://t.me/joinchat/G2mpk7-S85eM7sb7"])
    else:
        from .funcList import menu_table

        {
            "1": menu_table["basic"],
            "2": menu_table["system"],
            "3": menu_table["net"],
            "4": menu_table["api"],
            "5": menu_table["image"],
            "6": onlineDocuments,
        }[res[0]]()



from .. import headers, user_lang
from . import pre_check
import requests
import json

lolicon_token = pre_check("lolicon_token", False)


def loli_img(keyword: str = ""):
    try:
        res = requests.get(
            "https://api.lolicon.app/setu/",
            headers=headers,
            params={
                "apikey": lolicon_token if lolicon_token else "",
                "r18": "2",
                "keyword": keyword,
                "num": 1,
                "proxy": "disable",
            },
        )
    except Exception as e:
        return False, repr(e), False
    else:
        if res.status_code != requests.codes.ok:
            return (
                False,
                ("Get Data Failed" if user_lang != "zh" else "获取图源数据失败"),
                False,
            )
        data = json.loads(res.text)
        return data["code"] == 0, data["msg"], data["data"]


def magnet2torrent(hash: str):
    try:
        from ..NetTools import get_fileinfo
        from ..NetTools.NormalDL import normal_dl

        infos = get_fileinfo("https://m2t.lolicon.app/m/" + hash)
        normal_dl(infos[0], set_name=f'{infos[-1].headers["torrent-name"]}.torrent')
        return True, "Success"
    except Exception as e:
        return False, repr(e)



from setuptools import setup, find_packages

with open("README.md", "r") as fh:
    long_description = fh.read()

VERSION = "0.6.38"

setup(
    name="QuickStart_Rhy",
    version=VERSION,
    description="Simplify the operation in terminal!",
    long_description=long_description,
    long_description_content_type="text/markdown",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    keywords="Simplify the operation in terminal!",
    author="RhythmLian",
    url="https://github.com/Rhythmicc/qs",
    license="MIT",
    packages=find_packages(),
    include_package_data=True,
    zip_safe=True,
    install_requires=[
        "requests",
        "urllib3",
        "rarfile",
        "rich",
        "py7zr",
        "inquirer-rhy",
    ],
    entry_points={"console_scripts": ["qs = QuickStart_Rhy.main:main"]},
)



# coding=utf-8
"""
功能映射与菜单提示

Function mapping and menu prompt
"""
from . import user_lang, qs_default_console
from .qsLesson import lesson


def color_rep(ss, is_top_menu: bool = False):
    """
    格式化菜单文本（添加颜色）

    Format menu text (add color)

    :param is_top_menu:
    :param ss: 文本
    :return: 处理后的文本
    """
    from rich.table import Table, Column
    from rich.box import SIMPLE

    ss = ss.strip().split("\n")

    tb = Table(
        *(
            [
                Column("1", style="bold magenta", justify="left"),
                Column("2", style="bold yellow", justify="center"),
                Column("3", style="bold green", justify="right"),
            ]
            if is_top_menu
            else [
                Column("1", style="bold magenta", justify="left"),
                Column("2", style="bold yellow", justify="right"),
            ]
        ),
        title=f'[bold underline]{"QuickStart_Rhy" if is_top_menu else ss[0].strip().strip(":")}'
        + "\n",
        show_header=False,
        show_edge=False,
        box=SIMPLE,
    )
    for i, line in enumerate(ss if is_top_menu else ss[1:]):
        if ":->" in line:
            line = [i.strip() for i in line.split(":->")]
            if "|" in line[1]:
                line += [i.strip() for i in line.pop(-1).split("|")]
        tb.add_row(*line)
    qs_default_console.print(tb, justify="center")


def base_menu():
    """基础类菜单 | Basic menu"""
    color_rep(
        """Basic Tools help:
    qs u   <url...>          :-> open urls using default browser
    qs a   <app> \[file..]    :-> open app or open file by app ()
    qs f   <file...>         :-> open file by default app
    qs cal <exp>             :-> calculate exp
    qs time                  :-> view current time
    qs pact                  :-> output string in clipboard
    qs fcopy <file>          :-> copy file content to clipboard
    qs copy  <file>          :-> copy file to clipboard
    qs play  <file...>       :-> Play Musics
    qs lp    <file...>       :-> Print Files
    qs sas                   :-> Switch Audio Source ()"""
    ) if user_lang != "zh" else color_rep(
        """基础工具:
    qs u   <链接...>          :-> 使用默认浏览器打开多个链接
    qs a   <应用> \[文件...]   :-> 打开应用或使用应用打开文件 ()
    qs f   <文件...>          :-> 使用合适的应用打开文件
    qs cal <表达式>            :-> 计算算术表达式
    qs time                  :-> 查看当前时间
    qs pcat                  :-> 输出粘贴板内容
    qs fcopy <文件>           :-> 拷贝文件内容到粘贴板
    qs copy  <文件>           :-> 拷贝文件到粘贴板
    qs play  <文件...>        :-> 播放音乐
    qs lp    <文件...>        :-> 打印文件
    qs sas                   :-> 切换音频源 ()"""
    )


def system_menu():
    """系统类菜单 | System menu"""
    color_rep(
        """System Tools help:
    qs top                   :-> cpu and memory monitor
    qs clear                 :-> free memory
    qs go2git                :-> go to the webpage of the git config
    qs mount                 :-> mount dmg ()
    qs unmount               :-> unmount dmg ()
    qs mktar  <path...>      :-> create gzipped archive for path
    qs untar  <path...>      :-> extract *.tar.*
    qs mkzip  <path...>      :-> make a zip for path
    qs unzip  <path...>      :-> extract *.zip file
    qs unrar  <path...>      :-> extract *.rar file
    qs mk7z   <path...>      :-> make a 7z archive for path
    qs un7z   <path...>      :-> extract *.7z file
    qs md5    <path...>      :-> calculate md5 of files
    qs sha1   <path...>      :-> calculate SHA1 of files
    qs sha256 <path...>      :-> calculate SHA256 of files
    qs sha512 <path...>      :-> calculate SHA512 of files
    qs diff <dir1> <dir2>    :-> diff two directories"""
    ) if user_lang != "zh" else color_rep(
        """系统工具:
    qs top                   :-> CPU和内存监控器
    qs clear                 :-> 清理本机内存
    qs go2git                :-> 自动前往git目录托管仓库的网页
    qs mount                 :-> 挂载 dmg ()
    qs unmount               :-> 退出 dmg ()
    qs mktar  <路径...>      :-> 使用多个文件或文件夹创建tar压缩包
    qs untar  <路径...>      :-> 解压各种格式的tar包
    qs mkzip  <路径...>      :-> 使用多个文件或文件夹创建zip压缩包
    qs unzip  <路径...>      :-> 解压zip压缩包
    qs unrar  <路径...>      :-> 解压rar压缩包
    qs mk7z   <路径...>      :-> 使用多个文件或文件夹创建7z 压缩包
    qs un7z   <路径...>      :-> 解压7z 压缩包
    qs md5    <路径...>      :-> 计算文件的md5值
    qs sha1   <路径...>      :-> 计算文件sha1值
    qs sha256 <路径...>      :-> 计算文件sha256值
    qs sha512 <路径...>      :-> 计算文件sha512值
    qs diff <目录1> <目录2>    :-> 对比两个文件夹差异"""
    )


def net_menu():
    """网络类菜单 | Network menu"""
    color_rep(
        """Net Tools help:
    qs http \[ip] [-bind url] :-> start a multithread ftp server
    qs netinfo <domains...*> :-> get url's info which in clipboard or params 
    qs dl [urls] [-help]     :-> download file from url(in clipboard)
    qs wifi                  :-> connect wifi
    qs upgrade               :-> update qs"""
    ) if user_lang != "zh" else color_rep(
        """网络工具:
    qs http \[IP] [-bind 链接] :-> 在当前路径下开启多线程http服务
    qs netinfo <域名...*> :-> 获取命令参数或剪切板中链接或ip的信息 
    qs dl <链接*>             :-> 从命令参数或剪切板中链接下载文件
    qs wifi                  :-> 连接wifi
    qs upgrade               :-> 更新qs"""
    )


def api_menu():
    """api类菜单 | API menu"""
    color_rep(
        """API Tools help:
    qs trans  \[content]       :-> translate the content(in clipboard)
    qs pinyin \[content]      :-> get the pinyin of Chinese content
    qs rmbg   <img>            :-> remove image background
    qs smms   <img/*.md>       :-> upload img or img in markdown to sm.ms
    qs alioss --help          :-> get aliyun oss api help menu
    qs txcos  --help          :-> get tencent cos api help menu
    qs qiniu  --help          :-> get qiniu oss api help menu
    qs weather \[address]     :-> check weather (of address)
    qs LG  <image>            :-> make image larger(with AI)
    qs nlp \[words]           :-> Text(or in clipboard) error correction
    qs bcv   <url/video code> :-> get Bilibili video cover image with <url>
    qs gbc   <garbage...>     :-> check Chinese garbage classification
    qs svi   <url*>      :-> get short video info (show url)
    qs svd   <url*>            :-> download short video info as mp4
    qs acg   [--save]          :-> get an acg image link (or save)
    qs bing  [--save]          :-> get an bing image link (or save)
    qs phi   <url> [--save]   :-> get <image url> in url (preview on Mac)
    qs kd    <courier number> :-> Query China express
    qs loli  [--save] [-p]    :-> get and loli image link [or save (use default proxy)]
    qs setu  [--save] [-p]    :-> randomly call acg, acg2, loli
    qs exc  <num> <to> \[fr]  :-> Query <number> <to> corresponding <number?> <fr>
    qs zhihu                 :-> Get zhihu.com Daily
    qs wallhaven [--save]     :-> Get Wallhaven Toplist
    qs lmgtfy <keywords>     :-> Get LMGTFY link for keywords
    qs d60 [--save]          :-> Get the daily 60-second morning report
    qs m2t <-f path | -u link | *> :-> transform magnet link to torrent file
    qs d2m <designation> :-> search magnet url by designation
    qs doutu <keywords> :-> get expression package of keywords
    qs joke :-> get a Chinese joke"""
    ) if user_lang != "zh" else color_rep(
        """API工具:
    qs trans  \[内容]       :-> 翻译命令参数或剪切板中的内容
    qs pinyin \[内容]      :-> 获取中文内容的拼音
    qs rmbg   <图像>            :-> 移除图片背景
    qs smms   <图像/*.md>       :-> 将图片或Markdown中图片上传至sm.ms
    qs alioss --help          :-> 获取阿里云对象存储的使用帮助
    qs txcos  --help          :-> 获取腾讯云对象存储的使用帮助
    qs qiniu  --help          :-> 获取七牛云对象存储的使用帮助
    qs weather \[城市拼音]     :-> 获取当地天气（或指定地址天气）
    qs LG  <图像>            :-> 通过百度图像效果增强放大图片
    qs nlp \[words]           :-> 通过百度NLP进行文本纠错
    qs bcv   <链接/视频号> :-> 获取B站视频、直播封面图片
    qs gbc   <垃圾名称...>     :-> 查询垃圾分类
    qs svi   <链接*>      :-> 获取多平台短视频信息 (展示链接)
    qs svd   <链接*>            :-> 下载多平台短视频为mp4
    qs acg   [--save]          :-> 获取一张acg图片链接 (或保存)
    qs bing  [--save]          :-> 获取一张bing图片链接 (或保存)
    qs phi   <链接> [--save]   :-> 获取<链接>里的图片链接 (Mac上iTerm可预览)
    qs kd    <快递单号> :-> 查询国内快递
    qs loli  [--save] [-p]     :-> 获取一张萝莉图片链接 [或(使用默认代理)下载]
    qs setu  [--save] [-p]     :-> 随机调用acg, acg2, loli
    qs exc <数量> <to> \[fr]   :-> 汇率查询<数量> <to币种> 对应 多少 <fr币种>
    qs zhihu                :-> 获取知乎日报
    qs wallhaven [--save]     :-> 获取Wallhaven Top动漫涩图
    qs lmgtfy <关键词>     :-> 获取关键词的LMGTFY链接
    qs d60 [--save]           :-> 获取每日60秒早报
    qs m2t <-f 路径 | -u 链接 | *> :-> 转换磁力链接为种子文件
    qs d2m <番号> :-> 搜索番号的磁力链
    qs doutu <关键词> :-> 获取关键词的表情包
    qs joke :-> 获取一个中文笑话"""
    )


def image_menu():
    """图像处理类菜单 | Image Deal Menu"""
    color_rep(
        """Image Tools help:
    qs stbg <pic> <to> \[from]    :-> color replace for picture
    qs icat <img>            :-> preview image on terminal
    qs v2gif <path> \[sz] \[fps] :-> generate gif from video
    qs v2mp4 <video>         :-> format video to mp4
    qs v2mp3 <video>         :-> extract audio from video and save in MP3
    qs rmaudio <video>       :-> remove audio in video (return mp4 only)
    qs i2png <imgs...>       :-> transform imgs to png
    qs i2jpg <imgs...>       :-> transform imgs to jpg
    qs fmti <pic> <to> \[exp...] :-> format image to <to> color except \[exp...] color
    qs vsta    <video> <audio> :-> set video's audio"""
    ) if user_lang != "zh" else color_rep(
        """图像处理:
    qs stbg    <图像> <to> \[from]    :-> 替换图片颜色 (from默认为透明)
    qs icat    <图像>            :-> 在终端预览图片
    qs v2gif   <路径> \[sz] \[fps] :-> 将视频导出为gif
    qs v2mp4   <视频>         :-> 将视频导出为mp4
    qs v2mp3   <视频>         :-> 提取视频音频为mp3
    qs rmaudio <视频>       :-> 删除mp4文件音频
    qs i2png   <图像...>       :-> 将图像转换为png
    qs i2jpg   <图像...>       :-> 将图像转换为jpg
    qs fmti    <图像> <to> \[exp...] :-> 格式化图片颜色为<to>，忽略\[exp...]颜色
    qs vsta    <视频> <音频> :-> 设置视频的音频"""
    )


menu_table = {
    "basic": base_menu,
    "system": system_menu,
    "net": net_menu,
    "api": api_menu,
    "image": image_menu,
    "lesson": lesson,
}

basic_funcs = {
    "self": "basic",
    "u": "open_url",
    "a": "open_app",
    "f": "open_file",
    "time": "cur_time",
    "cal": "calculate",
    "pcat": "pcat",
    "fcopy": "fcopy",
    "user_lang": "get_user_lang",
    "play": "play_music",
    "copy": "copy",
    "lp": "qs_print",
    "sas": "sas",
}

system_funcs = {
    "self": "system",
    "top": "top",
    "clear": "clear_mem",
    "go2git": "go_github",
    "mktar": "mktar",
    "untar": "untar",
    "mkzip": "mkzip",
    "unzip": "unzip",
    "unrar": "unrar",
    "mk7z": "mk7z",
    "un7z": "un7z",
    "md5": "md5",
    "sha1": "sha1",
    "sha256": "sha256",
    "sha512": "sha512",
    "diff": "diff_dir",
    "mount": "mount_dmg",
    "unmount": "unmount_dmg",
}

net_funcs = {
    "self": "netTools",
    "http": "http",
    "dl": "download",
    "wifi": "wifi",
    "upgrade": "upgrade",
    "netinfo": "netinfo",
}

api_funcs = {
    "self": "api",
    "trans": "translate",
    "pinyin": "pinyin",
    "rmbg": "remove_bg",
    "smms": "smms",
    "alioss": "ali_oss",
    "qiniu": "qiniu",
    "txcos": "txcos",
    "LG": "largeImage",
    "weather": "weather",
    "nlp": "AipNLP",
    "cb": "CommonClipboard",
    "bcv": "bili_cover",
    "gbc": "gbc",
    "svi": "short_video_info",
    "svd": "short_video_dl",
    "acg": "acg",
    "bing": "bingImg",
    "phi": "preview_html_images",
    "kd": "kdCheck",
    "loli": "loli",
    "photo": "photo",
    "setu": "setu",
    "exc": "exchange",
    "zhihu": "zhihuDaily",
    "wallhaven": "wallhaven",
    "lmgtfy": "lmgtfy",
    "d60": "daily60s",
    "m2t": "m2t",
    "d2m": "d2m",
    "doutu": "doutu",
    "joke": "joke",
    "gpt": "gpt",
}

image_funcs = {
    "self": "imageDeal",
    "stbg": "set_img_background",
    "icat": "icat",
    "v2gif": "v2gif",
    "rmaudio": "remove_audio",
    "v2mp4": "v2mp4",
    "v2mp3": "v2mp3",
    "i2png": "i2png",
    "i2jpg": "i2jpg",
    "fmti": "fmt_img_color",
    "vsta": "vsta",
}



# coding=utf-8
"""
系统工具库

System tool library
"""


def clear_mem():
    """
    清理系统内存

    Clean system memory

    :return: None
    """
    from .. import dir_char, system, external_exec

    if dir_char == "\\":
        print("Not support")
    else:
        if system.startswith("darwin"):
            external_exec("sudo purge")
        else:
            external_exec("sync")
            external_exec("echo 3 > /proc/sys/vm/drop_caches")


def get_core_num():
    """
    获取核数量

    :return: core num
    """
    from .. import requirePackage

    return requirePackage("psutil").cpu_count()
